@@ -1,6 +1,7 @@
 """
 Form Widget classes specific to the Django admin site.
 """
+
 import copy
 import json
 
@@ -8,7 +9,7 @@ from django import forms
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.core.validators import URLValidator
-from django.db.models import CASCADE
+from django.db.models import CASCADE, UUIDField
 from django.urls import reverse
 from django.urls.exceptions import NoReverseMatch
 from django.utils.html import smart_urlquote
@@ -48,7 +49,7 @@ class FilteredSelectMultiple(forms.SelectMultiple):
         return context
 
 
-class AdminDateWidget(forms.DateInput):
+class BaseAdminDateWidget(forms.DateInput):
     class Media:
         js = [
             "admin/js/calendar.js",
@@ -60,7 +61,11 @@ class AdminDateWidget(forms.DateInput):
         super().__init__(attrs=attrs, format=format)
 
 
-class AdminTimeWidget(forms.TimeInput):
+class AdminDateWidget(BaseAdminDateWidget):
+    template_name = "admin/widgets/date.html"
+
+
+class BaseAdminTimeWidget(forms.TimeInput):
     class Media:
         js = [
             "admin/js/calendar.js",
@@ -72,6 +77,10 @@ class AdminTimeWidget(forms.TimeInput):
         super().__init__(attrs=attrs, format=format)
 
 
+class AdminTimeWidget(BaseAdminTimeWidget):
+    template_name = "admin/widgets/time.html"
+
+
 class AdminSplitDateTime(forms.SplitDateTimeWidget):
     """
     A SplitDateTime Widget that has some admin-specific styling.
@@ -80,7 +89,7 @@ class AdminSplitDateTime(forms.SplitDateTimeWidget):
     template_name = "admin/widgets/split_datetime.html"
 
     def __init__(self, attrs=None):
-        widgets = [AdminDateWidget, AdminTimeWidget]
+        widgets = [BaseAdminDateWidget, BaseAdminTimeWidget]
         # Note that we're calling MultiWidget, not SplitDateTimeWidget, because
         # we want to define widgets.
         forms.MultiWidget.__init__(self, widgets, attrs)
@@ -137,7 +146,7 @@ class ForeignKeyRawIdWidget(forms.TextInput):
     def get_context(self, name, value, attrs):
         context = super().get_context(name, value, attrs)
         rel_to = self.rel.model
-        if rel_to in self.admin_site._registry:
+        if self.admin_site.is_registered(rel_to):
             # The related object is registered with the same AdminSite
             related_url = reverse(
                 "admin:%s_%s_changelist"
@@ -154,7 +163,10 @@ class ForeignKeyRawIdWidget(forms.TextInput):
             context["related_url"] = related_url
             context["link_title"] = _("Lookup")
             # The JavaScript code looks for this class.
-            context["widget"]["attrs"].setdefault("class", "vForeignKeyRawIdAdminField")
+            css_class = "vForeignKeyRawIdAdminField"
+            if isinstance(self.rel.get_related_field(), UUIDField):
+                css_class += " vUUIDField"
+            context["widget"]["attrs"].setdefault("class", css_class)
         else:
             context["related_url"] = None
         if context["widget"]["value"]:
@@ -211,7 +223,7 @@ class ManyToManyRawIdWidget(ForeignKeyRawIdWidget):
 
     def get_context(self, name, value, attrs):
         context = super().get_context(name, value, attrs)
-        if self.rel.model in self.admin_site._registry:
+        if self.admin_site.is_registered(self.rel.model):
             # The related object is registered with the same AdminSite
             context["widget"]["attrs"]["class"] = "vManyToManyRawIdAdminField"
         return context
@@ -251,13 +263,12 @@ class RelatedFieldWidgetWrapper(forms.Widget):
     ):
         self.needs_multipart_form = widget.needs_multipart_form
         self.attrs = widget.attrs
-        self.choices = widget.choices
         self.widget = widget
         self.rel = rel
         # Backwards compatible check for whether a user can add related
         # objects.
         if can_add_related is None:
-            can_add_related = rel.model in admin_site._registry
+            can_add_related = admin_site.is_registered(rel.model)
         self.can_add_related = can_add_related
         # XXX: The UX does not support multiple selected values.
         multiple = getattr(widget, "allow_multiple_selected", False)
@@ -284,6 +295,14 @@ class RelatedFieldWidgetWrapper(forms.Widget):
     def media(self):
         return self.widget.media
 
+    @property
+    def choices(self):
+        return self.widget.choices
+
+    @choices.setter
+    def choices(self, value):
+        self.widget.choices = value
+
     def get_related_url(self, info, action, *args):
         return reverse(
             "admin:%s_%s_%s" % (info + (action,)),
@@ -296,11 +315,11 @@ class RelatedFieldWidgetWrapper(forms.Widget):
 
         rel_opts = self.rel.model._meta
         info = (rel_opts.app_label, rel_opts.model_name)
-        self.widget.choices = self.choices
+        related_field_name = self.rel.get_related_field().name
         url_params = "&".join(
             "%s=%s" % param
             for param in [
-                (TO_FIELD_VAR, self.rel.get_related_field().name),
+                (TO_FIELD_VAR, related_field_name),
                 (IS_POPUP_VAR, 1),
             ]
         )
@@ -314,6 +333,7 @@ class RelatedFieldWidgetWrapper(forms.Widget):
             "can_change_related": self.can_change_related,
             "can_delete_related": self.can_delete_related,
             "can_view_related": self.can_view_related,
+            "model_has_limit_choices_to": self.rel.limit_choices_to,
         }
         if self.can_add_related:
             context["add_related_url"] = self.get_related_url(info, "add")
@@ -322,6 +342,7 @@ class RelatedFieldWidgetWrapper(forms.Widget):
                 info, "delete", "__fk__"
             )
         if self.can_view_related or self.can_change_related:
+            context["view_related_url_params"] = f"{TO_FIELD_VAR}={related_field_name}"
             context["change_related_template_url"] = self.get_related_url(
                 info, "change", "__fk__"
             )
@@ -369,7 +390,7 @@ class AdminURLFieldWidget(forms.URLInput):
         context["current_label"] = _("Currently:")
         context["change_label"] = _("Change:")
         context["widget"]["href"] = (
-            smart_urlquote(context["widget"]["value"]) if value else ""
+            smart_urlquote(context["widget"]["value"]) if url_valid else ""
         )
         context["url_valid"] = url_valid
         return context
@@ -447,6 +468,19 @@ SELECT2_TRANSLATIONS = {
 SELECT2_TRANSLATIONS.update({"zh-hans": "zh-CN", "zh-hant": "zh-TW"})
 
 
+def get_select2_language():
+    lang_code = get_language()
+    supported_code = SELECT2_TRANSLATIONS.get(lang_code)
+    if supported_code is None and lang_code is not None:
+        # If 'zh-hant-tw' is not supported, try subsequent language codes i.e.
+        # 'zh-hant' and 'zh'.
+        i = None
+        while (i := lang_code.rfind("-", 0, i)) > -1:
+            if supported_code := SELECT2_TRANSLATIONS.get(lang_code[:i]):
+                return supported_code
+    return supported_code
+
+
 class AutocompleteMixin:
     """
     Select widget mixin that loads options from AutocompleteJsonView via AJAX.
@@ -463,7 +497,7 @@ class AutocompleteMixin:
         self.db = using
         self.choices = choices
         self.attrs = {} if attrs is None else attrs.copy()
-        self.i18n_name = SELECT2_TRANSLATIONS.get(get_language())
+        self.i18n_name = get_select2_language()
 
     def get_url(self):
         return reverse(self.url_name % self.admin_site.name)
