@@ -1,13 +1,16 @@
+import warnings
 from enum import Enum
+from types import NoneType
 
 from django.core.exceptions import FieldError, ValidationError
 from django.db import connections
-from django.db.models.expressions import Exists, ExpressionList, F
+from django.db.models.expressions import Exists, ExpressionList, F, OrderBy
 from django.db.models.indexes import IndexExpression
-from django.db.models.lookups import Exact
+from django.db.models.lookups import Exact, IsNull
 from django.db.models.query_utils import Q
 from django.db.models.sql.query import Query
 from django.db.utils import DEFAULT_DB_ALIAS
+from django.utils.deprecation import RemovedInDjango60Warning
 from django.utils.translation import gettext_lazy as _
 
 __all__ = ["BaseConstraint", "CheckConstraint", "Deferrable", "UniqueConstraint"]
@@ -15,14 +18,40 @@ __all__ = ["BaseConstraint", "CheckConstraint", "Deferrable", "UniqueConstraint"
 
 class BaseConstraint:
     default_violation_error_message = _("Constraint “%(name)s” is violated.")
+    violation_error_code = None
     violation_error_message = None
 
-    def __init__(self, name, violation_error_message=None):
+    # RemovedInDjango60Warning: When the deprecation ends, replace with:
+    # def __init__(
+    #     self, *, name, violation_error_code=None, violation_error_message=None
+    # ):
+    def __init__(
+        self, *args, name=None, violation_error_code=None, violation_error_message=None
+    ):
+        # RemovedInDjango60Warning.
+        if name is None and not args:
+            raise TypeError(
+                f"{self.__class__.__name__}.__init__() missing 1 required keyword-only "
+                f"argument: 'name'"
+            )
         self.name = name
+        if violation_error_code is not None:
+            self.violation_error_code = violation_error_code
         if violation_error_message is not None:
             self.violation_error_message = violation_error_message
         else:
             self.violation_error_message = self.default_violation_error_message
+        # RemovedInDjango60Warning.
+        if args:
+            warnings.warn(
+                f"Passing positional arguments to {self.__class__.__name__} is "
+                f"deprecated.",
+                RemovedInDjango60Warning,
+                stacklevel=2,
+            )
+            for arg, attr in zip(args, ["name", "violation_error_message"]):
+                if arg:
+                    setattr(self, attr, arg)
 
     @property
     def contains_expressions(self):
@@ -52,6 +81,8 @@ class BaseConstraint:
             and self.violation_error_message != self.default_violation_error_message
         ):
             kwargs["violation_error_message"] = self.violation_error_message
+        if self.violation_error_code is not None:
+            kwargs["violation_error_code"] = self.violation_error_code
         return (path, (), kwargs)
 
     def clone(self):
@@ -60,13 +91,19 @@ class BaseConstraint:
 
 
 class CheckConstraint(BaseConstraint):
-    def __init__(self, *, check, name, violation_error_message=None):
+    def __init__(
+        self, *, check, name, violation_error_code=None, violation_error_message=None
+    ):
         self.check = check
         if not getattr(check, "conditional", False):
             raise TypeError(
                 "CheckConstraint.check must be a Q instance or boolean expression."
             )
-        super().__init__(name, violation_error_message=violation_error_message)
+        super().__init__(
+            name=name,
+            violation_error_code=violation_error_code,
+            violation_error_message=violation_error_message,
+        )
 
     def _get_check_sql(self, model, schema_editor):
         query = Query(model=model, alias_cols=False)
@@ -87,18 +124,31 @@ class CheckConstraint(BaseConstraint):
         return schema_editor._delete_check_sql(model, self.name)
 
     def validate(self, model, instance, exclude=None, using=DEFAULT_DB_ALIAS):
-        against = instance._get_field_value_map(meta=model._meta, exclude=exclude)
+        against = instance._get_field_expression_map(meta=model._meta, exclude=exclude)
         try:
             if not Q(self.check).check(against, using=using):
-                raise ValidationError(self.get_violation_error_message())
+                raise ValidationError(
+                    self.get_violation_error_message(), code=self.violation_error_code
+                )
         except FieldError:
             pass
 
     def __repr__(self):
-        return "<%s: check=%s name=%s>" % (
+        return "<%s: check=%s name=%s%s%s>" % (
             self.__class__.__qualname__,
             self.check,
             repr(self.name),
+            (
+                ""
+                if self.violation_error_code is None
+                else " violation_error_code=%r" % self.violation_error_code
+            ),
+            (
+                ""
+                if self.violation_error_message is None
+                or self.violation_error_message == self.default_violation_error_message
+                else " violation_error_message=%r" % self.violation_error_message
+            ),
         )
 
     def __eq__(self, other):
@@ -106,6 +156,7 @@ class CheckConstraint(BaseConstraint):
             return (
                 self.name == other.name
                 and self.check == other.check
+                and self.violation_error_code == other.violation_error_code
                 and self.violation_error_message == other.violation_error_message
             )
         return super().__eq__(other)
@@ -135,6 +186,8 @@ class UniqueConstraint(BaseConstraint):
         deferrable=None,
         include=None,
         opclasses=(),
+        nulls_distinct=None,
+        violation_error_code=None,
         violation_error_message=None,
     ):
         if not name:
@@ -148,7 +201,7 @@ class UniqueConstraint(BaseConstraint):
             raise ValueError(
                 "UniqueConstraint.fields and expressions are mutually exclusive."
             )
-        if not isinstance(condition, (type(None), Q)):
+        if not isinstance(condition, (NoneType, Q)):
             raise ValueError("UniqueConstraint.condition must be a Q instance.")
         if condition and deferrable:
             raise ValueError("UniqueConstraint with conditions cannot be deferred.")
@@ -163,14 +216,16 @@ class UniqueConstraint(BaseConstraint):
                 "UniqueConstraint.opclasses cannot be used with expressions. "
                 "Use django.contrib.postgres.indexes.OpClass() instead."
             )
-        if not isinstance(deferrable, (type(None), Deferrable)):
-            raise ValueError(
+        if not isinstance(deferrable, (NoneType, Deferrable)):
+            raise TypeError(
                 "UniqueConstraint.deferrable must be a Deferrable instance."
             )
-        if not isinstance(include, (type(None), list, tuple)):
-            raise ValueError("UniqueConstraint.include must be a list or tuple.")
+        if not isinstance(include, (NoneType, list, tuple)):
+            raise TypeError("UniqueConstraint.include must be a list or tuple.")
         if not isinstance(opclasses, (list, tuple)):
-            raise ValueError("UniqueConstraint.opclasses must be a list or tuple.")
+            raise TypeError("UniqueConstraint.opclasses must be a list or tuple.")
+        if not isinstance(nulls_distinct, (NoneType, bool)):
+            raise TypeError("UniqueConstraint.nulls_distinct must be a bool.")
         if opclasses and len(fields) != len(opclasses):
             raise ValueError(
                 "UniqueConstraint.fields and UniqueConstraint.opclasses must "
@@ -181,11 +236,16 @@ class UniqueConstraint(BaseConstraint):
         self.deferrable = deferrable
         self.include = tuple(include) if include else ()
         self.opclasses = opclasses
+        self.nulls_distinct = nulls_distinct
         self.expressions = tuple(
             F(expression) if isinstance(expression, str) else expression
             for expression in expressions
         )
-        super().__init__(name, violation_error_message=violation_error_message)
+        super().__init__(
+            name=name,
+            violation_error_code=violation_error_code,
+            violation_error_message=violation_error_message,
+        )
 
     @property
     def contains_expressions(self):
@@ -228,6 +288,7 @@ class UniqueConstraint(BaseConstraint):
             include=include,
             opclasses=self.opclasses,
             expressions=expressions,
+            nulls_distinct=self.nulls_distinct,
         )
 
     def create_sql(self, model, schema_editor):
@@ -246,6 +307,7 @@ class UniqueConstraint(BaseConstraint):
             include=include,
             opclasses=self.opclasses,
             expressions=expressions,
+            nulls_distinct=self.nulls_distinct,
         )
 
     def remove_sql(self, model, schema_editor):
@@ -262,10 +324,11 @@ class UniqueConstraint(BaseConstraint):
             include=include,
             opclasses=self.opclasses,
             expressions=expressions,
+            nulls_distinct=self.nulls_distinct,
         )
 
     def __repr__(self):
-        return "<%s:%s%s%s%s%s%s%s>" % (
+        return "<%s:%s%s%s%s%s%s%s%s%s%s>" % (
             self.__class__.__qualname__,
             "" if not self.fields else " fields=%s" % repr(self.fields),
             "" if not self.expressions else " expressions=%s" % repr(self.expressions),
@@ -274,6 +337,22 @@ class UniqueConstraint(BaseConstraint):
             "" if self.deferrable is None else " deferrable=%r" % self.deferrable,
             "" if not self.include else " include=%s" % repr(self.include),
             "" if not self.opclasses else " opclasses=%s" % repr(self.opclasses),
+            (
+                ""
+                if self.nulls_distinct is None
+                else " nulls_distinct=%r" % self.nulls_distinct
+            ),
+            (
+                ""
+                if self.violation_error_code is None
+                else " violation_error_code=%r" % self.violation_error_code
+            ),
+            (
+                ""
+                if self.violation_error_message is None
+                or self.violation_error_message == self.default_violation_error_message
+                else " violation_error_message=%r" % self.violation_error_message
+            ),
         )
 
     def __eq__(self, other):
@@ -286,6 +365,8 @@ class UniqueConstraint(BaseConstraint):
                 and self.include == other.include
                 and self.opclasses == other.opclasses
                 and self.expressions == other.expressions
+                and self.nulls_distinct is other.nulls_distinct
+                and self.violation_error_code == other.violation_error_code
                 and self.violation_error_message == other.violation_error_message
             )
         return super().__eq__(other)
@@ -302,6 +383,8 @@ class UniqueConstraint(BaseConstraint):
             kwargs["include"] = self.include
         if self.opclasses:
             kwargs["opclasses"] = self.opclasses
+        if self.nulls_distinct is not None:
+            kwargs["nulls_distinct"] = self.nulls_distinct
         return path, self.expressions, kwargs
 
     def validate(self, model, instance, exclude=None, using=DEFAULT_DB_ALIAS):
@@ -313,9 +396,15 @@ class UniqueConstraint(BaseConstraint):
                     return
                 field = model._meta.get_field(field_name)
                 lookup_value = getattr(instance, field.attname)
-                if lookup_value is None or (
-                    lookup_value == ""
-                    and connections[using].features.interprets_empty_strings_as_nulls
+                if (
+                    self.nulls_distinct is not False
+                    and lookup_value is None
+                    or (
+                        lookup_value == ""
+                        and connections[
+                            using
+                        ].features.interprets_empty_strings_as_nulls
+                    )
                 ):
                     # A composite constraint containing NULL value cannot cause
                     # a violation since NULL != NULL in SQL.
@@ -326,38 +415,55 @@ class UniqueConstraint(BaseConstraint):
             # Ignore constraints with excluded fields.
             if exclude:
                 for expression in self.expressions:
-                    for expr in expression.flatten():
-                        if isinstance(expr, F) and expr.name in exclude:
-                            return
-            replacement_map = instance._get_field_value_map(
-                meta=model._meta, exclude=exclude
-            )
-            expressions = [
-                Exact(expr, expr.replace_references(replacement_map))
-                for expr in self.expressions
-            ]
-            queryset = queryset.filter(*expressions)
+                    if hasattr(expression, "flatten"):
+                        for expr in expression.flatten():
+                            if isinstance(expr, F) and expr.name in exclude:
+                                return
+                    elif isinstance(expression, F) and expression.name in exclude:
+                        return
+            replacements = {
+                F(field): value
+                for field, value in instance._get_field_expression_map(
+                    meta=model._meta, exclude=exclude
+                ).items()
+            }
+            filters = []
+            for expr in self.expressions:
+                # Ignore ordering.
+                if isinstance(expr, OrderBy):
+                    expr = expr.expression
+                rhs = expr.replace_expressions(replacements)
+                condition = Exact(expr, rhs)
+                if self.nulls_distinct is False:
+                    condition = Q(condition) | Q(IsNull(expr, True), IsNull(rhs, True))
+                filters.append(condition)
+            queryset = queryset.filter(*filters)
         model_class_pk = instance._get_pk_val(model._meta)
         if not instance._state.adding and model_class_pk is not None:
             queryset = queryset.exclude(pk=model_class_pk)
         if not self.condition:
             if queryset.exists():
-                if self.expressions:
-                    raise ValidationError(self.get_violation_error_message())
-                # When fields are defined, use the unique_error_message() for
-                # backward compatibility.
-                for model, constraints in instance.get_constraints():
-                    for constraint in constraints:
-                        if constraint is self:
-                            raise ValidationError(
-                                instance.unique_error_message(model, self.fields)
-                            )
+                if self.fields:
+                    # When fields are defined, use the unique_error_message() for
+                    # backward compatibility.
+                    raise ValidationError(
+                        instance.unique_error_message(model, self.fields),
+                    )
+                raise ValidationError(
+                    self.get_violation_error_message(),
+                    code=self.violation_error_code,
+                )
         else:
-            against = instance._get_field_value_map(meta=model._meta, exclude=exclude)
+            against = instance._get_field_expression_map(
+                meta=model._meta, exclude=exclude
+            )
             try:
                 if (self.condition & Exists(queryset.filter(self.condition))).check(
                     against, using=using
                 ):
-                    raise ValidationError(self.get_violation_error_message())
+                    raise ValidationError(
+                        self.get_violation_error_message(),
+                        code=self.violation_error_code,
+                    )
             except FieldError:
                 pass
