@@ -1,24 +1,28 @@
 """Default variable filters."""
+
 import random as random_module
 import re
 import types
-from decimal import ROUND_HALF_UP, Context, Decimal, InvalidOperation
+import warnings
+from decimal import ROUND_HALF_UP, Context, Decimal, InvalidOperation, getcontext
 from functools import wraps
+from inspect import unwrap
 from operator import itemgetter
 from pprint import pformat
 from urllib.parse import quote
 
 from django.utils import formats
 from django.utils.dateformat import format, time_format
+from django.utils.deprecation import RemovedInDjango51Warning
 from django.utils.encoding import iri_to_uri
-from django.utils.html import (
-    avoid_wrapping, conditional_escape, escape, escapejs,
-    json_script as _json_script, linebreaks, strip_tags, urlize as _urlize,
-)
+from django.utils.html import avoid_wrapping, conditional_escape, escape, escapejs
+from django.utils.html import json_script as _json_script
+from django.utils.html import linebreaks, strip_tags
+from django.utils.html import urlize as _urlize
 from django.utils.safestring import SafeData, mark_safe
-from django.utils.text import (
-    Truncator, normalize_newlines, phone2numeric, slugify as _slugify, wrap,
-)
+from django.utils.text import Truncator, normalize_newlines, phone2numeric
+from django.utils.text import slugify as _slugify
+from django.utils.text import wrap
 from django.utils.timesince import timesince, timeuntil
 from django.utils.translation import gettext, ngettext
 
@@ -32,31 +36,29 @@ register = Library()
 # STRING DECORATOR    #
 #######################
 
+
 def stringfilter(func):
     """
     Decorator for filters which should only receive strings. The object
     passed as the first positional argument will be converted to a string.
     """
-    def _dec(*args, **kwargs):
-        args = list(args)
-        args[0] = str(args[0])
-        if (isinstance(args[0], SafeData) and
-                getattr(_dec._decorated_function, 'is_safe', False)):
-            return mark_safe(func(*args, **kwargs))
-        return func(*args, **kwargs)
 
-    # Include a reference to the real function (used to check original
-    # arguments by the template parser, and to bear the 'is_safe' attribute
-    # when multiple decorators are applied).
-    _dec._decorated_function = getattr(func, '_decorated_function', func)
+    @wraps(func)
+    def _dec(first, *args, **kwargs):
+        first = str(first)
+        result = func(first, *args, **kwargs)
+        if isinstance(first, SafeData) and getattr(unwrap(func), "is_safe", False):
+            result = mark_safe(result)
+        return result
 
-    return wraps(func)(_dec)
+    return _dec
 
 
 ###################
 # STRINGS         #
 ###################
 
+
 @register.filter(is_safe=True)
 @stringfilter
 def addslashes(value):
@@ -65,7 +67,7 @@ def addslashes(value):
     example. Less useful for escaping JavaScript; use the ``escapejs``
     filter instead.
     """
-    return value.replace('\\', '\\\\').replace('"', '\\"').replace("'", "\\'")
+    return value.replace("\\", "\\\\").replace('"', '\\"').replace("'", "\\'")
 
 
 @register.filter(is_safe=True)
@@ -83,10 +85,10 @@ def escapejs_filter(value):
 
 
 @register.filter(is_safe=True)
-def json_script(value, element_id):
+def json_script(value, element_id=None):
     """
     Output value JSON-encoded, wrapped in a <script type="application/json">
-    tag.
+    tag (with an optional id).
     """
     return _json_script(value, element_id)
 
@@ -139,41 +141,56 @@ def floatformat(text, arg=-1):
     use_l10n = True
     if isinstance(arg, str):
         last_char = arg[-1]
-        if arg[-2:] in {'gu', 'ug'}:
+        if arg[-2:] in {"gu", "ug"}:
             force_grouping = True
             use_l10n = False
             arg = arg[:-2] or -1
-        elif last_char == 'g':
+        elif last_char == "g":
             force_grouping = True
             arg = arg[:-1] or -1
-        elif last_char == 'u':
+        elif last_char == "u":
             use_l10n = False
             arg = arg[:-1] or -1
     try:
-        input_val = repr(text)
+        input_val = str(text)
         d = Decimal(input_val)
     except InvalidOperation:
         try:
             d = Decimal(str(float(text)))
         except (ValueError, InvalidOperation, TypeError):
-            return ''
+            return ""
     try:
         p = int(arg)
     except ValueError:
         return input_val
 
+    _, digits, exponent = d.as_tuple()
+    try:
+        number_of_digits_and_exponent_sum = len(digits) + abs(exponent)
+    except TypeError:
+        # Exponent values can be "F", "n", "N".
+        number_of_digits_and_exponent_sum = 0
+
+    # Values with more than 200 digits, or with a large exponent, are returned "as is"
+    # to avoid high memory consumption and potential denial-of-service attacks.
+    # The cut-off of 200 is consistent with django.utils.numberformat.floatformat().
+    if number_of_digits_and_exponent_sum > 200:
+        return input_val
+
     try:
         m = int(d) - d
     except (ValueError, OverflowError, InvalidOperation):
         return input_val
 
-    if not m and p < 0:
-        return mark_safe(formats.number_format(
-            '%d' % (int(d)),
-            0,
-            use_l10n=use_l10n,
-            force_grouping=force_grouping,
-        ))
+    if not m and p <= 0:
+        return mark_safe(
+            formats.number_format(
+                "%d" % (int(d)),
+                0,
+                use_l10n=use_l10n,
+                force_grouping=force_grouping,
+            )
+        )
 
     exp = Decimal(1).scaleb(-abs(p))
     # Set the precision high enough to avoid an exception (#15789).
@@ -181,6 +198,7 @@ def floatformat(text, arg=-1):
     units = len(tupl[1])
     units += -tupl[2] if m else tupl[2]
     prec = abs(p) + units + 1
+    prec = max(getcontext().prec, prec)
 
     # Avoid conversion to scientific notation by accessing `sign`, `digits`,
     # and `exponent` from Decimal.as_tuple() directly.
@@ -188,17 +206,19 @@ def floatformat(text, arg=-1):
     sign, digits, exponent = rounded_d.as_tuple()
     digits = [str(digit) for digit in reversed(digits)]
     while len(digits) <= abs(exponent):
-        digits.append('0')
-    digits.insert(-exponent, '.')
+        digits.append("0")
+    digits.insert(-exponent, ".")
     if sign and rounded_d:
-        digits.append('-')
-    number = ''.join(reversed(digits))
-    return mark_safe(formats.number_format(
-        number,
-        abs(p),
-        use_l10n=use_l10n,
-        force_grouping=force_grouping,
-    ))
+        digits.append("-")
+    number = "".join(reversed(digits))
+    return mark_safe(
+        formats.number_format(
+            number,
+            abs(p),
+            use_l10n=use_l10n,
+            force_grouping=force_grouping,
+        )
+    )
 
 
 @register.filter(is_safe=True)
@@ -212,7 +232,7 @@ def iriencode(value):
 @stringfilter
 def linenumbers(value, autoescape=True):
     """Display text with line numbers."""
-    lines = value.split('\n')
+    lines = value.split("\n")
     # Find the maximum width of the line count, for use with zero padding
     # string format command
     width = str(len(str(len(lines))))
@@ -222,7 +242,7 @@ def linenumbers(value, autoescape=True):
     else:
         for i, line in enumerate(lines):
             lines[i] = ("%0" + width + "d. %s") % (i + 1, escape(line))
-    return mark_safe('\n'.join(lines))
+    return mark_safe("\n".join(lines))
 
 
 @register.filter(is_safe=True)
@@ -279,7 +299,7 @@ def stringformat(value, arg):
 def title(value):
     """Convert a string into titlecase."""
     t = re.sub("([a-z])'([A-Z])", lambda m: m[0].lower(), value.title())
-    return re.sub(r'\d([A-Z])', lambda m: m[0].lower(), t)
+    return re.sub(r"\d([A-Z])", lambda m: m[0].lower(), t)
 
 
 @register.filter(is_safe=True)
@@ -318,7 +338,7 @@ def truncatewords(value, arg):
         length = int(arg)
     except ValueError:  # Invalid literal for int().
         return value  # Fail silently.
-    return Truncator(value).words(length, truncate=' …')
+    return Truncator(value).words(length, truncate=" …")
 
 
 @register.filter(is_safe=True)
@@ -332,7 +352,7 @@ def truncatewords_html(value, arg):
         length = int(arg)
     except ValueError:  # invalid literal for int()
         return value  # Fail silently.
-    return Truncator(value).words(length, html=True, truncate=' …')
+    return Truncator(value).words(length, html=True, truncate=" …")
 
 
 @register.filter(is_safe=False)
@@ -355,7 +375,7 @@ def urlencode(value, safe=None):
     """
     kwargs = {}
     if safe is not None:
-        kwargs['safe'] = safe
+        kwargs["safe"] = safe
     return quote(value, **kwargs)
 
 
@@ -375,7 +395,9 @@ def urlizetrunc(value, limit, autoescape=True):
 
     Argument: Length to truncate URLs to.
     """
-    return mark_safe(_urlize(value, trim_url_limit=int(limit), nofollow=True, autoescape=autoescape))
+    return mark_safe(
+        _urlize(value, trim_url_limit=int(limit), nofollow=True, autoescape=autoescape)
+    )
 
 
 @register.filter(is_safe=False)
@@ -418,8 +440,8 @@ def center(value, arg):
 def cut(value, arg):
     """Remove all values of arg from the given string."""
     safe = isinstance(value, SafeData)
-    value = value.replace(arg, '')
-    if safe and arg != ';':
+    value = value.replace(arg, "")
+    if safe and arg != ";":
         return mark_safe(value)
     return value
 
@@ -428,6 +450,7 @@ def cut(value, arg):
 # HTML STRINGS    #
 ###################
 
+
 @register.filter("escape", is_safe=True)
 @stringfilter
 def escape_filter(value):
@@ -435,6 +458,16 @@ def escape_filter(value):
     return conditional_escape(value)
 
 
+@register.filter(is_safe=True)
+def escapeseq(value):
+    """
+    An "escape" filter for sequences. Mark each element in the sequence,
+    individually, as a string that should be auto-escaped. Return a list with
+    the results.
+    """
+    return [conditional_escape(obj) for obj in value]
+
+
 @register.filter(is_safe=True)
 @stringfilter
 def force_escape(value):
@@ -469,7 +502,7 @@ def linebreaksbr(value, autoescape=True):
     value = normalize_newlines(value)
     if autoescape:
         value = escape(value)
-    return mark_safe(value.replace('\n', '<br>'))
+    return mark_safe(value.replace("\n", "<br>"))
 
 
 @register.filter(is_safe=True)
@@ -500,6 +533,7 @@ def striptags(value):
 # LISTS           #
 ###################
 
+
 def _property_resolver(arg):
     """
     When arg is convertible to float, behave like operator.itemgetter(arg)
@@ -521,8 +555,8 @@ def _property_resolver(arg):
     try:
         float(arg)
     except ValueError:
-        if VARIABLE_ATTRIBUTE_SEPARATOR + '_' in arg or arg[0] == '_':
-            raise AttributeError('Access to private variables is forbidden.')
+        if VARIABLE_ATTRIBUTE_SEPARATOR + "_" in arg or arg[0] == "_":
+            raise AttributeError("Access to private variables is forbidden.")
         parts = arg.split(VARIABLE_ATTRIBUTE_SEPARATOR)
 
         def resolve(value):
@@ -547,7 +581,7 @@ def dictsort(value, arg):
     try:
         return sorted(value, key=_property_resolver(arg))
     except (AttributeError, TypeError):
-        return ''
+        return ""
 
 
 @register.filter(is_safe=False)
@@ -559,7 +593,7 @@ def dictsortreversed(value, arg):
     try:
         return sorted(value, key=_property_resolver(arg), reverse=True)
     except (AttributeError, TypeError):
-        return ''
+        return ""
 
 
 @register.filter(is_safe=False)
@@ -568,7 +602,7 @@ def first(value):
     try:
         return value[0]
     except IndexError:
-        return ''
+        return ""
 
 
 @register.filter(is_safe=True, needs_autoescape=True)
@@ -576,8 +610,9 @@ def join(value, arg, autoescape=True):
     """Join a list with a string, like Python's ``str.join(list)``."""
     try:
         if autoescape:
-            value = [conditional_escape(v) for v in value]
-        data = conditional_escape(arg).join(value)
+            data = conditional_escape(arg).join([conditional_escape(v) for v in value])
+        else:
+            data = arg.join(value)
     except TypeError:  # Fail silently if arg isn't iterable.
         return value
     return mark_safe(data)
@@ -589,7 +624,7 @@ def last(value):
     try:
         return value[-1]
     except IndexError:
-        return ''
+        return ""
 
 
 @register.filter(is_safe=False)
@@ -604,16 +639,24 @@ def length(value):
 @register.filter(is_safe=False)
 def length_is(value, arg):
     """Return a boolean of whether the value's length is the argument."""
+    warnings.warn(
+        "The length_is template filter is deprecated in favor of the length template "
+        "filter and the == operator within an {% if %} tag.",
+        RemovedInDjango51Warning,
+    )
     try:
         return len(value) == int(arg)
     except (ValueError, TypeError):
-        return ''
+        return ""
 
 
 @register.filter(is_safe=True)
 def random(value):
     """Return a random item from the list."""
-    return random_module.choice(value)
+    try:
+        return random_module.choice(value)
+    except IndexError:
+        return ""
 
 
 @register.filter("slice", is_safe=True)
@@ -623,7 +666,7 @@ def slice_filter(value, arg):
     """
     try:
         bits = []
-        for x in str(arg).split(':'):
+        for x in str(arg).split(":"):
             if not x:
                 bits.append(None)
             else:
@@ -659,6 +702,7 @@ def unordered_list(value, autoescape=True):
     if autoescape:
         escaper = conditional_escape
     else:
+
         def escaper(x):
             return x
 
@@ -687,16 +731,19 @@ def unordered_list(value, autoescape=True):
             pass
 
     def list_formatter(item_list, tabs=1):
-        indent = '\t' * tabs
+        indent = "\t" * tabs
         output = []
         for item, children in walk_items(item_list):
-            sublist = ''
+            sublist = ""
             if children:
-                sublist = '\n%s<ul>\n%s\n%s</ul>\n%s' % (
-                    indent, list_formatter(children, tabs + 1), indent, indent)
-            output.append('%s<li>%s%s</li>' % (
-                indent, escaper(item), sublist))
-        return '\n'.join(output)
+                sublist = "\n%s<ul>\n%s\n%s</ul>\n%s" % (
+                    indent,
+                    list_formatter(children, tabs + 1),
+                    indent,
+                    indent,
+                )
+            output.append("%s<li>%s%s</li>" % (indent, escaper(item), sublist))
+        return "\n".join(output)
 
     return mark_safe(list_formatter(value))
 
@@ -705,6 +752,7 @@ def unordered_list(value, autoescape=True):
 # INTEGERS        #
 ###################
 
+
 @register.filter(is_safe=False)
 def add(value, arg):
     """Add the arg to the value."""
@@ -714,7 +762,7 @@ def add(value, arg):
         try:
             return value + arg
         except Exception:
-            return ''
+            return ""
 
 
 @register.filter(is_safe=False)
@@ -742,62 +790,64 @@ def get_digit(value, arg):
 # DATES           #
 ###################
 
+
 @register.filter(expects_localtime=True, is_safe=False)
 def date(value, arg=None):
     """Format a date according to the given format."""
-    if value in (None, ''):
-        return ''
+    if value in (None, ""):
+        return ""
     try:
         return formats.date_format(value, arg)
     except AttributeError:
         try:
             return format(value, arg)
         except AttributeError:
-            return ''
+            return ""
 
 
 @register.filter(expects_localtime=True, is_safe=False)
 def time(value, arg=None):
     """Format a time according to the given format."""
-    if value in (None, ''):
-        return ''
+    if value in (None, ""):
+        return ""
     try:
         return formats.time_format(value, arg)
     except (AttributeError, TypeError):
         try:
             return time_format(value, arg)
         except (AttributeError, TypeError):
-            return ''
+            return ""
 
 
 @register.filter("timesince", is_safe=False)
 def timesince_filter(value, arg=None):
     """Format a date as the time since that date (i.e. "4 days, 6 hours")."""
     if not value:
-        return ''
+        return ""
     try:
         if arg:
             return timesince(value, arg)
         return timesince(value)
     except (ValueError, TypeError):
-        return ''
+        return ""
 
 
 @register.filter("timeuntil", is_safe=False)
 def timeuntil_filter(value, arg=None):
     """Format a date as the time until that date (i.e. "4 days, 6 hours")."""
     if not value:
-        return ''
+        return ""
     try:
         return timeuntil(value, arg)
     except (ValueError, TypeError):
-        return ''
+        return ""
 
 
 ###################
 # LOGIC           #
 ###################
 
+
 @register.filter(is_safe=False)
 def default(value, arg):
     """If value is unavailable, use given default."""
@@ -836,8 +886,8 @@ def yesno(value, arg=None):
     """
     if arg is None:
         # Translators: Please do not add spaces around commas.
-        arg = gettext('yes,no,maybe')
-    bits = arg.split(',')
+        arg = gettext("yes,no,maybe")
+    bits = arg.split(",")
     if len(bits) < 2:
         return value  # Invalid arg.
     try:
@@ -856,6 +906,7 @@ def yesno(value, arg=None):
 # MISC            #
 ###################
 
+
 @register.filter(is_safe=True)
 def filesizeformat(bytes_):
     """
@@ -865,7 +916,7 @@ def filesizeformat(bytes_):
     try:
         bytes_ = int(bytes_)
     except (TypeError, ValueError, UnicodeDecodeError):
-        value = ngettext("%(size)d byte", "%(size)d bytes", 0) % {'size': 0}
+        value = ngettext("%(size)d byte", "%(size)d bytes", 0) % {"size": 0}
         return avoid_wrapping(value)
 
     def filesize_number_format(value):
@@ -882,7 +933,7 @@ def filesizeformat(bytes_):
         bytes_ = -bytes_  # Allow formatting of negative numbers.
 
     if bytes_ < KB:
-        value = ngettext("%(size)d byte", "%(size)d bytes", bytes_) % {'size': bytes_}
+        value = ngettext("%(size)d byte", "%(size)d bytes", bytes_) % {"size": bytes_}
     elif bytes_ < MB:
         value = gettext("%s KB") % filesize_number_format(bytes_ / KB)
     elif bytes_ < GB:
@@ -900,7 +951,7 @@ def filesizeformat(bytes_):
 
 
 @register.filter(is_safe=False)
-def pluralize(value, arg='s'):
+def pluralize(value, arg="s"):
     """
     Return a plural suffix if the value is not 1, '1', or an object of
     length 1. By default, use 's' as the suffix:
@@ -922,11 +973,11 @@ def pluralize(value, arg='s'):
     * If value is 1, cand{{ value|pluralize:"y,ies" }} display "candy".
     * If value is 2, cand{{ value|pluralize:"y,ies" }} display "candies".
     """
-    if ',' not in arg:
-        arg = ',' + arg
-    bits = arg.split(',')
+    if "," not in arg:
+        arg = "," + arg
+    bits = arg.split(",")
     if len(bits) > 2:
-        return ''
+        return ""
     singular_suffix, plural_suffix = bits[:2]
 
     try:
@@ -938,7 +989,7 @@ def pluralize(value, arg='s'):
             return singular_suffix if len(value) == 1 else plural_suffix
         except TypeError:  # len() of unsized object.
             pass
-    return ''
+    return ""
 
 
 @register.filter("phone2numeric", is_safe=True)
