@@ -12,6 +12,8 @@
 import sys
 from os.path import abspath, dirname, join
 
+from sphinx import version_info as sphinx_version
+
 # Workaround for sphinx-build recursion limit overflow:
 # pickle.dump(doctree, f, pickle.HIGHEST_PROTOCOL)
 #  RuntimeError: maximum recursion depth exceeded while pickling an object
@@ -54,8 +56,8 @@ autosectionlabel_maxdepth = 2
 linkcheck_ignore = [
     # Special-use addresses and domain names. (RFC 6761/6890)
     r"^https?://(?:127\.0\.0\.1|\[::1\])(?::\d+)?/",
-    r"^https?://(?:[^/\.]+\.)*example\.(?:com|net|org)(?::\d+)?/",
-    r"^https?://(?:[^/\.]+\.)*(?:example|invalid|localhost|test)(?::\d+)?/",
+    r"^https?://(?:[^/.]+\.)*example\.(?:com|net|org)(?::\d+)?/",
+    r"^https?://(?:[^/.]+\.)*(?:example|invalid|localhost|test)(?::\d+)?/",
     # Pages that are inaccessible because they require authentication.
     r"^https://github\.com/[^/]+/[^/]+/fork",
     r"^https://code\.djangoproject\.com/github/login",
@@ -111,7 +113,7 @@ copyright = "Django Software Foundation and contributors"
 # built documents.
 #
 # The short X.Y version.
-version = "4.1"
+version = "5.0"
 # The full version, including alpha/beta/rc tags.
 try:
     from django import VERSION, get_version
@@ -128,17 +130,20 @@ else:
     release = django_release()
 
 # The "development version" of Django
-django_next_version = "4.2"
+django_next_version = "5.1"
 
 extlinks = {
     "bpo": ("https://bugs.python.org/issue?@action=redirect&bpo=%s", "bpo-%s"),
     "commit": ("https://github.com/django/django/commit/%s", "%s"),
-    "cve": ("https://nvd.nist.gov/vuln/detail/CVE-%s", "CVE-%s"),
+    "pypi": ("https://pypi.org/project/%s/", "%s"),
     # A file or directory. GitHub redirects from blob to tree if needed.
     "source": ("https://github.com/django/django/blob/main/%s", "%s"),
     "ticket": ("https://code.djangoproject.com/ticket/%s", "#%s"),
 }
 
+if sphinx_version < (8, 1):
+    extlinks["cve"] = ("https://www.cve.org/CVERecord?id=CVE-%s", "CVE-%s")
+
 # The language for content autogenerated by Sphinx. Refer to documentation
 # for a list of supported languages.
 # language = None
@@ -178,7 +183,7 @@ pygments_style = "trac"
 intersphinx_mapping = {
     "python": ("https://docs.python.org/3/", None),
     "sphinx": ("https://www.sphinx-doc.org/en/master/", None),
-    "psycopg2": ("https://www.psycopg.org/docs/", None),
+    "psycopg": ("https://www.psycopg.org/psycopg3/docs/", None),
 }
 
 # Python's docs don't change every week.
@@ -404,7 +409,7 @@ epub_cover = ("", "epub-cover.html")
 # The format is a list of tuples containing the path and title.
 # epub_pre_files = []
 
-# HTML files shat should be inserted after the pages created by sphinx.
+# HTML files that should be inserted after the pages created by sphinx.
 # The format is a list of tuples containing the path and title.
 # epub_post_files = []
 
