@@ -3,7 +3,7 @@ from django.contrib.auth.admin import UserAdmin
 from django.contrib.auth.models import User
 from django.core.paginator import Paginator
 
-from .models import Band, Child, Event, Parent, Swallow
+from .models import Band, Child, Event, GrandChild, Parent, ProxyUser, Swallow
 
 site = admin.AdminSite(name="admin")
 
@@ -53,6 +53,13 @@ class ChildAdmin(admin.ModelAdmin):
         return super().get_queryset(request).select_related("parent")
 
 
+class GrandChildAdmin(admin.ModelAdmin):
+    list_display = ["name"]
+
+
+site.register(GrandChild, GrandChildAdmin)
+
+
 class CustomPaginationAdmin(ChildAdmin):
     paginator = CustomPaginator
 
@@ -185,3 +192,24 @@ class EmptyValueChildAdmin(admin.ModelAdmin):
     @admin.display(empty_value="&dagger;")
     def age_display(self, obj):
         return obj.age
+
+
+class UnescapedTitleFilter(admin.SimpleListFilter):
+    title = "It's OK"
+    parameter_name = "is_active"
+
+    def lookups(self, request, model_admin):
+        return [("yes", "yes"), ("no", "no")]
+
+    def queryset(self, request, queryset):
+        if self.value() == "yes":
+            return queryset.filter(is_active=True)
+        else:
+            return queryset.filter(is_active=False)
+
+
+class CustomUserAdmin(UserAdmin):
+    list_filter = [UnescapedTitleFilter]
+
+
+site.register(ProxyUser, CustomUserAdmin)
