@@ -353,13 +353,13 @@ class TestInline(TestDataMixin, TestCase):
         )
         # The div containing the position field is hidden.
         self.assertInHTML(
-            '<div class="fieldBox field-position hidden">'
+            '<div class="flex-container fieldBox field-position hidden">'
             '<label class="inline">Position:</label>'
             '<div class="readonly">0</div></div>',
             response.rendered_content,
         )
         self.assertInHTML(
-            '<div class="fieldBox field-position hidden">'
+            '<div class="flex-container fieldBox field-position hidden">'
             '<label class="inline">Position:</label>'
             '<div class="readonly">1</div></div>',
             response.rendered_content,
@@ -382,14 +382,14 @@ class TestInline(TestDataMixin, TestCase):
         # The whole line containing position field is hidden.
         self.assertInHTML(
             '<div class="form-row hidden field-position">'
-            "<div><label>Position:</label>"
-            '<div class="readonly">0</div></div></div>',
+            '<div><div class="flex-container"><label>Position:</label>'
+            '<div class="readonly">0</div></div></div></div>',
             response.rendered_content,
         )
         self.assertInHTML(
             '<div class="form-row hidden field-position">'
-            "<div><label>Position:</label>"
-            '<div class="readonly">1</div></div></div>',
+            '<div><div class="flex-container"><label>Position:</label>'
+            '<div class="readonly">1</div></div></div></div>',
             response.rendered_content,
         )
 
@@ -452,7 +452,7 @@ class TestInline(TestDataMixin, TestCase):
         self.assertContains(
             response,
             '<input id="id_-1-0-name" type="text" class="vTextField" name="-1-0-name" '
-            'maxlength="100">',
+            'maxlength="100" aria-describedby="id_-1-0-name_helptext">',
             html=True,
         )
         self.assertContains(
@@ -1303,7 +1303,10 @@ class TestReadOnlyChangeViewInlinePermissions(TestCase):
             '<a href="/admin/admin_inlines/poll/" class="closelink">Close</a>',
             html=True,
         )
-        delete_link = '<p class="deletelink-box"><a href="/admin/admin_inlines/poll/%s/delete/" class="deletelink">Delete</a></p>'  # noqa
+        delete_link = (
+            '<a href="/admin/admin_inlines/poll/%s/delete/" class="deletelink">Delete'
+            "</a>"
+        )
         self.assertNotContains(response, delete_link % self.poll.id, html=True)
         self.assertNotContains(
             response,
@@ -1904,11 +1907,9 @@ class SeleniumTests(AdminSeleniumTestCase):
             "border-top-%s",
         ]
         for prop in border_properties:
-            prop = prop % "width"
-            self.assertEqual(element.value_of_css_property(prop), width)
+            self.assertEqual(element.value_of_css_property(prop % "width"), width)
         for prop in border_properties:
-            prop = prop % "style"
-            self.assertEqual(element.value_of_css_property(prop), style)
+            self.assertEqual(element.value_of_css_property(prop % "style"), style)
         # Convert hex color to rgb.
         self.assertRegex(color, "#[0-9a-f]{6}")
         r, g, b = int(color[1:3], 16), int(color[3:5], 16), int(color[5:], 16)
@@ -1919,8 +1920,7 @@ class SeleniumTests(AdminSeleniumTestCase):
             "rgba(%d, %d, %d, 1)" % (r, g, b),
         ]
         for prop in border_properties:
-            prop = prop % "color"
-            self.assertIn(element.value_of_css_property(prop), colors)
+            self.assertIn(element.value_of_css_property(prop % "color"), colors)
 
     def test_inline_formset_error_input_border(self):
         from selenium.webdriver.common.by import By
