@@ -119,6 +119,7 @@ from .models import (
     Simple,
     Sketch,
     Song,
+    Square,
     State,
     Story,
     StumpJoke,
@@ -215,6 +216,7 @@ class ArticleAdmin(ArticleAdminWithExtraUrl):
         "model_month",
         "order_by_f_expression",
         "order_by_orderby_expression",
+        "model_property_is_from_past",
     )
     list_editable = ("section",)
     list_filter = ("date", "section")
@@ -679,11 +681,13 @@ class ReadOnlyPizzaAdmin(admin.ModelAdmin):
 class WorkHourAdmin(admin.ModelAdmin):
     list_display = ("datum", "employee")
     list_filter = ("employee",)
+    show_facets = admin.ShowFacets.ALWAYS
 
 
 class FoodDeliveryAdmin(admin.ModelAdmin):
     list_display = ("reference", "driver", "restaurant")
     list_editable = ("driver", "restaurant")
+    show_facets = admin.ShowFacets.NEVER
 
 
 class CoverLetterAdmin(admin.ModelAdmin):
@@ -1165,6 +1169,18 @@ class GetFormsetsArgumentCheckingAdmin(admin.ModelAdmin):
         return super().get_formsets_with_inlines(request, obj)
 
 
+class CountryAdmin(admin.ModelAdmin):
+    search_fields = ["name"]
+
+
+class TravelerAdmin(admin.ModelAdmin):
+    autocomplete_fields = ["living_country"]
+
+
+class SquareAdmin(admin.ModelAdmin):
+    readonly_fields = ("area",)
+
+
 site = admin.AdminSite(name="admin")
 site.site_url = "/my-site-url/"
 site.register(Article, ArticleAdmin)
@@ -1286,8 +1302,9 @@ site.register(ExplicitlyProvidedPK, GetFormsetsArgumentCheckingAdmin)
 site.register(ImplicitlyGeneratedPK, GetFormsetsArgumentCheckingAdmin)
 site.register(UserProxy)
 site.register(Box)
-site.register(Country)
-site.register(Traveler)
+site.register(Country, CountryAdmin)
+site.register(Traveler, TravelerAdmin)
+site.register(Square, SquareAdmin)
 
 # Register core models we need in our tests
 site.register(User, UserAdmin)
