@@ -3,6 +3,7 @@ import json
 from contextlib import contextmanager
 
 from django.contrib import admin
+from django.contrib.admin.exceptions import NotRegistered
 from django.contrib.admin.tests import AdminSeleniumTestCase
 from django.contrib.admin.views.autocomplete import AutocompleteJsonView
 from django.contrib.auth.models import Permission, User
@@ -61,8 +62,11 @@ site.register(Toy, autocomplete_fields=["child"])
 
 @contextmanager
 def model_admin(model, model_admin, admin_site=site):
-    org_admin = admin_site._registry.get(model)
-    if org_admin:
+    try:
+        org_admin = admin_site.get_model_admin(model)
+    except NotRegistered:
+        org_admin = None
+    else:
         admin_site.unregister(model)
     admin_site.register(model, model_admin)
     try:
@@ -408,13 +412,14 @@ class SeleniumTests(AdminSeleniumTestCase):
     @contextmanager
     def select2_ajax_wait(self, timeout=10):
         from selenium.common.exceptions import NoSuchElementException
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.support import expected_conditions as ec
 
         yield
         with self.disable_implicit_wait():
             try:
-                loading_element = self.selenium.find_element_by_css_selector(
-                    "li.select2-results__option.loading-results"
+                loading_element = self.selenium.find_element(
+                    By.CSS_SELECTOR, "li.select2-results__option.loading-results"
                 )
             except NoSuchElementException:
                 pass
@@ -422,65 +427,73 @@ class SeleniumTests(AdminSeleniumTestCase):
                 self.wait_until(ec.staleness_of(loading_element), timeout=timeout)
 
     def test_select(self):
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.common.keys import Keys
         from selenium.webdriver.support.ui import Select
 
         self.selenium.get(
             self.live_server_url + reverse("autocomplete_admin:admin_views_answer_add")
         )
-        elem = self.selenium.find_element_by_css_selector(".select2-selection")
-        elem.click()  # Open the autocomplete dropdown.
-        results = self.selenium.find_element_by_css_selector(".select2-results")
+        elem = self.selenium.find_element(By.CSS_SELECTOR, ".select2-selection")
+        with self.select2_ajax_wait():
+            elem.click()  # Open the autocomplete dropdown.
+        results = self.selenium.find_element(By.CSS_SELECTOR, ".select2-results")
         self.assertTrue(results.is_displayed())
-        option = self.selenium.find_element_by_css_selector(".select2-results__option")
+        option = self.selenium.find_element(By.CSS_SELECTOR, ".select2-results__option")
         self.assertEqual(option.text, "No results found")
-        elem.click()  # Close the autocomplete dropdown.
+        with self.select2_ajax_wait():
+            elem.click()  # Close the autocomplete dropdown.
         q1 = Question.objects.create(question="Who am I?")
         Question.objects.bulk_create(
             Question(question=str(i)) for i in range(PAGINATOR_SIZE + 10)
         )
-        elem.click()  # Reopen the dropdown now that some objects exist.
-        result_container = self.selenium.find_element_by_css_selector(
-            ".select2-results"
+        with self.select2_ajax_wait():
+            elem.click()  # Reopen the dropdown now that some objects exist.
+        result_container = self.selenium.find_element(
+            By.CSS_SELECTOR, ".select2-results"
         )
         self.assertTrue(result_container.is_displayed())
-        results = result_container.find_elements_by_css_selector(
-            ".select2-results__option"
-        )
         # PAGINATOR_SIZE results and "Loading more results".
-        self.assertEqual(len(results), PAGINATOR_SIZE + 1)
-        search = self.selenium.find_element_by_css_selector(".select2-search__field")
+        self.assertCountSeleniumElements(
+            ".select2-results__option",
+            PAGINATOR_SIZE + 1,
+            root_element=result_container,
+        )
+        search = self.selenium.find_element(By.CSS_SELECTOR, ".select2-search__field")
         # Load next page of results by scrolling to the bottom of the list.
         with self.select2_ajax_wait():
-            for _ in range(len(results)):
+            for _ in range(PAGINATOR_SIZE + 1):
                 search.send_keys(Keys.ARROW_DOWN)
-        results = result_container.find_elements_by_css_selector(
-            ".select2-results__option"
-        )
         # All objects are now loaded.
-        self.assertEqual(len(results), PAGINATOR_SIZE + 11)
+        self.assertCountSeleniumElements(
+            ".select2-results__option",
+            PAGINATOR_SIZE + 11,
+            root_element=result_container,
+        )
         # Limit the results with the search field.
         with self.select2_ajax_wait():
             search.send_keys("Who")
             # Ajax request is delayed.
             self.assertTrue(result_container.is_displayed())
-            results = result_container.find_elements_by_css_selector(
-                ".select2-results__option"
+            self.assertCountSeleniumElements(
+                ".select2-results__option",
+                PAGINATOR_SIZE + 12,
+                root_element=result_container,
             )
-            self.assertEqual(len(results), PAGINATOR_SIZE + 12)
         self.assertTrue(result_container.is_displayed())
-        results = result_container.find_elements_by_css_selector(
-            ".select2-results__option"
+        self.assertCountSeleniumElements(
+            ".select2-results__option", 1, root_element=result_container
         )
-        self.assertEqual(len(results), 1)
         # Select the result.
-        search.send_keys(Keys.RETURN)
-        select = Select(self.selenium.find_element_by_id("id_question"))
+        with self.select2_ajax_wait():
+            search.send_keys(Keys.RETURN)
+        select = Select(self.selenium.find_element(By.ID, "id_question"))
         self.assertEqual(
             select.first_selected_option.get_attribute("value"), str(q1.pk)
         )
 
     def test_select_multiple(self):
+        from selenium.webdriver.common.by import By
         from selenium.webdriver.common.keys import Keys
         from selenium.webdriver.support.ui import Select
 
@@ -488,66 +501,71 @@ class SeleniumTests(AdminSeleniumTestCase):
             self.live_server_url
             + reverse("autocomplete_admin:admin_views_question_add")
         )
-        elem = self.selenium.find_element_by_css_selector(".select2-selection")
-        elem.click()  # Open the autocomplete dropdown.
-        results = self.selenium.find_element_by_css_selector(".select2-results")
+        elem = self.selenium.find_element(By.CSS_SELECTOR, ".select2-selection")
+        with self.select2_ajax_wait():
+            elem.click()  # Open the autocomplete dropdown.
+        results = self.selenium.find_element(By.CSS_SELECTOR, ".select2-results")
         self.assertTrue(results.is_displayed())
-        option = self.selenium.find_element_by_css_selector(".select2-results__option")
+        option = self.selenium.find_element(By.CSS_SELECTOR, ".select2-results__option")
         self.assertEqual(option.text, "No results found")
-        elem.click()  # Close the autocomplete dropdown.
+        with self.select2_ajax_wait():
+            elem.click()  # Close the autocomplete dropdown.
         Question.objects.create(question="Who am I?")
         Question.objects.bulk_create(
             Question(question=str(i)) for i in range(PAGINATOR_SIZE + 10)
         )
-        elem.click()  # Reopen the dropdown now that some objects exist.
-        result_container = self.selenium.find_element_by_css_selector(
-            ".select2-results"
+        with self.select2_ajax_wait():
+            elem.click()  # Reopen the dropdown now that some objects exist.
+        result_container = self.selenium.find_element(
+            By.CSS_SELECTOR, ".select2-results"
         )
         self.assertTrue(result_container.is_displayed())
-        results = result_container.find_elements_by_css_selector(
-            ".select2-results__option"
+        self.assertCountSeleniumElements(
+            ".select2-results__option",
+            PAGINATOR_SIZE + 1,
+            root_element=result_container,
         )
-        self.assertEqual(len(results), PAGINATOR_SIZE + 1)
-        search = self.selenium.find_element_by_css_selector(".select2-search__field")
+        search = self.selenium.find_element(By.CSS_SELECTOR, ".select2-search__field")
         # Load next page of results by scrolling to the bottom of the list.
         with self.select2_ajax_wait():
-            for _ in range(len(results)):
+            for _ in range(PAGINATOR_SIZE + 1):
                 search.send_keys(Keys.ARROW_DOWN)
-        results = result_container.find_elements_by_css_selector(
-            ".select2-results__option"
+        self.assertCountSeleniumElements(
+            ".select2-results__option", 31, root_element=result_container
         )
-        self.assertEqual(len(results), 31)
         # Limit the results with the search field.
         with self.select2_ajax_wait():
             search.send_keys("Who")
             # Ajax request is delayed.
             self.assertTrue(result_container.is_displayed())
-            results = result_container.find_elements_by_css_selector(
-                ".select2-results__option"
+            self.assertCountSeleniumElements(
+                ".select2-results__option", 32, root_element=result_container
             )
-            self.assertEqual(len(results), 32)
         self.assertTrue(result_container.is_displayed())
-        results = result_container.find_elements_by_css_selector(
-            ".select2-results__option"
+
+        self.assertCountSeleniumElements(
+            ".select2-results__option", 1, root_element=result_container
         )
-        self.assertEqual(len(results), 1)
-        # Select the result.
-        search.send_keys(Keys.RETURN)
-        # Reopen the dropdown and add the first result to the selection.
-        elem.click()
-        search.send_keys(Keys.ARROW_DOWN)
-        search.send_keys(Keys.RETURN)
-        select = Select(self.selenium.find_element_by_id("id_related_questions"))
+        with self.select2_ajax_wait():
+            # Select the result.
+            search.send_keys(Keys.RETURN)
+            # Reopen the dropdown and add the first result to the selection.
+            elem.click()
+            search.send_keys(Keys.ARROW_DOWN)
+            search.send_keys(Keys.RETURN)
+        select = Select(self.selenium.find_element(By.ID, "id_related_questions"))
         self.assertEqual(len(select.all_selected_options), 2)
 
     def test_inline_add_another_widgets(self):
+        from selenium.webdriver.common.by import By
+
         def assertNoResults(row):
-            elem = row.find_element_by_css_selector(".select2-selection")
+            elem = row.find_element(By.CSS_SELECTOR, ".select2-selection")
             elem.click()  # Open the autocomplete dropdown.
-            results = self.selenium.find_element_by_css_selector(".select2-results")
+            results = self.selenium.find_element(By.CSS_SELECTOR, ".select2-results")
             self.assertTrue(results.is_displayed())
-            option = self.selenium.find_element_by_css_selector(
-                ".select2-results__option"
+            option = self.selenium.find_element(
+                By.CSS_SELECTOR, ".select2-results__option"
             )
             self.assertEqual(option.text, "No results found")
 
@@ -555,11 +573,11 @@ class SeleniumTests(AdminSeleniumTestCase):
         self.selenium.get(
             self.live_server_url + reverse("autocomplete_admin:admin_views_book_add")
         )
-        rows = self.selenium.find_elements_by_css_selector(".dynamic-authorship_set")
+        rows = self.selenium.find_elements(By.CSS_SELECTOR, ".dynamic-authorship_set")
         self.assertEqual(len(rows), 3)
         assertNoResults(rows[0])
         # Autocomplete works in rows added using the "Add another" button.
-        self.selenium.find_element_by_link_text("Add another Authorship").click()
-        rows = self.selenium.find_elements_by_css_selector(".dynamic-authorship_set")
+        self.selenium.find_element(By.LINK_TEXT, "Add another Authorship").click()
+        rows = self.selenium.find_elements(By.CSS_SELECTOR, ".dynamic-authorship_set")
         self.assertEqual(len(rows), 4)
         assertNoResults(rows[-1])
