@@ -49,7 +49,7 @@ def mock_inputs(inputs):
     """
 
     def inner(test_func):
-        def wrapped(*args):
+        def wrapper(*args):
             class mock_getpass:
                 @staticmethod
                 def getpass(prompt=b"Password: ", stream=None):
@@ -90,7 +90,7 @@ def mock_inputs(inputs):
                 createsuperuser.getpass = old_getpass
                 builtins.input = old_input
 
-        return wrapped
+        return wrapper
 
     return inner
 
@@ -312,6 +312,19 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
         # created password should be unusable
         self.assertFalse(u.has_usable_password())
 
+    def test_validate_username(self):
+        msg = (
+            "Enter a valid username. This value may contain only letters, numbers, "
+            "and @/./+/-/_ characters."
+        )
+        with self.assertRaisesMessage(CommandError, msg):
+            call_command(
+                "createsuperuser",
+                interactive=False,
+                username="🤠",
+                email="joe@somewhere.org",
+            )
+
     def test_non_ascii_verbose_name(self):
         @mock_inputs(
             {
@@ -955,6 +968,36 @@ class CreatesuperuserManagementCommandTestCase(TestCase):
                 stderr=new_io,
             )
 
+    def test_blank_email_allowed_non_interactive(self):
+        new_io = StringIO()
+
+        call_command(
+            "createsuperuser",
+            email="",
+            username="joe",
+            interactive=False,
+            stdout=new_io,
+            stderr=new_io,
+        )
+        self.assertEqual(new_io.getvalue().strip(), "Superuser created successfully.")
+        u = User.objects.get(username="joe")
+        self.assertEqual(u.email, "")
+
+    @mock.patch.dict(os.environ, {"DJANGO_SUPERUSER_EMAIL": ""})
+    def test_blank_email_allowed_non_interactive_environment_variable(self):
+        new_io = StringIO()
+
+        call_command(
+            "createsuperuser",
+            username="joe",
+            interactive=False,
+            stdout=new_io,
+            stderr=new_io,
+        )
+        self.assertEqual(new_io.getvalue().strip(), "Superuser created successfully.")
+        u = User.objects.get(username="joe")
+        self.assertEqual(u.email, "")
+
     def test_password_validation_bypass(self):
         """
         Password validation can be bypassed by entering 'y' at the prompt.
@@ -1472,3 +1515,22 @@ class CreatePermissionsTests(TestCase):
                 codename=codename,
             ).exists()
         )
+
+
+class DefaultDBRouter:
+    """Route all writes to default."""
+
+    def db_for_write(self, model, **hints):
+        return "default"
+
+
+@override_settings(DATABASE_ROUTERS=[DefaultDBRouter()])
+class CreatePermissionsMultipleDatabasesTests(TestCase):
+    databases = {"default", "other"}
+
+    def test_set_permissions_fk_to_using_parameter(self):
+        Permission.objects.using("other").delete()
+        with self.assertNumQueries(6, using="other") as captured_queries:
+            create_permissions(apps.get_app_config("auth"), verbosity=0, using="other")
+        self.assertIn("INSERT INTO", captured_queries[-1]["sql"].upper())
+        self.assertGreater(Permission.objects.using("other").count(), 0)
