@@ -1,14 +1,15 @@
 import unittest
+from unittest import mock
 
-from django.db import DatabaseError, connection
+from django.db import DatabaseError, NotSupportedError, connection
 from django.db.models import BooleanField
-from django.test import TransactionTestCase
+from django.test import TestCase, TransactionTestCase
 
 from ..models import Square, VeryLongModelNameZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
 
 
 @unittest.skipUnless(connection.vendor == "oracle", "Oracle tests")
-class Tests(unittest.TestCase):
+class Tests(TestCase):
     def test_quote_name(self):
         """'%' chars are escaped for query execution."""
         name = '"SOME%NAME"'
@@ -37,12 +38,6 @@ class Tests(unittest.TestCase):
             cursor.execute("BEGIN %s := 'X'; END; ", [var])
             self.assertEqual(var.getvalue(), "X")
 
-    def test_client_encoding(self):
-        """Client encoding is set correctly."""
-        connection.ensure_connection()
-        self.assertEqual(connection.connection.encoding, "UTF-8")
-        self.assertEqual(connection.connection.nencoding, "UTF-8")
-
     def test_order_of_nls_parameters(self):
         """
         An 'almost right' datetime works with configured NLS parameters
@@ -62,6 +57,17 @@ class Tests(unittest.TestCase):
                 field.set_attributes_from_name("is_nice")
                 self.assertIn('"IS_NICE" IN (0,1)', field.db_check(connection))
 
+    @mock.patch.object(
+        connection,
+        "get_database_version",
+        return_value=(18, 1),
+    )
+    def test_check_database_version_supported(self, mocked_get_database_version):
+        msg = "Oracle 19 or later is required (found 18.1)."
+        with self.assertRaisesMessage(NotSupportedError, msg):
+            connection.check_database_version_supported()
+        self.assertTrue(mocked_get_database_version.called)
+
 
 @unittest.skipUnless(connection.vendor == "oracle", "Oracle tests")
 class TransactionalTests(TransactionTestCase):
@@ -97,6 +103,8 @@ class TransactionalTests(TransactionTestCase):
                 cursor.execute('DROP TRIGGER "TRG_NO_DATA_FOUND"')
 
     def test_password_with_at_sign(self):
+        from django.db.backends.oracle.base import Database
+
         old_password = connection.settings_dict["PASSWORD"]
         connection.settings_dict["PASSWORD"] = "p@ssword"
         try:
@@ -104,8 +112,8 @@ class TransactionalTests(TransactionTestCase):
                 '/"p@ssword"@',
                 connection.client.connect_string(connection.settings_dict),
             )
-            with self.assertRaises(DatabaseError) as context:
-                connection.cursor()
+            with self.assertRaises(Database.DatabaseError) as context:
+                connection.connect()
             # Database exception: "ORA-01017: invalid username/password" is
             # expected.
             self.assertIn("ORA-01017", context.exception.args[0].message)
