@@ -144,35 +144,40 @@ class FileTests(unittest.TestCase):
         self.assertEqual(list(f), ["one\n", "two\n", "three"])
 
     def test_readable(self):
-        with tempfile.TemporaryFile() as temp, File(
-            temp, name="something.txt"
-        ) as test_file:
+        with (
+            tempfile.TemporaryFile() as temp,
+            File(temp, name="something.txt") as test_file,
+        ):
             self.assertTrue(test_file.readable())
         self.assertFalse(test_file.readable())
 
     def test_writable(self):
-        with tempfile.TemporaryFile() as temp, File(
-            temp, name="something.txt"
-        ) as test_file:
+        with (
+            tempfile.TemporaryFile() as temp,
+            File(temp, name="something.txt") as test_file,
+        ):
             self.assertTrue(test_file.writable())
         self.assertFalse(test_file.writable())
-        with tempfile.TemporaryFile("rb") as temp, File(
-            temp, name="something.txt"
-        ) as test_file:
+        with (
+            tempfile.TemporaryFile("rb") as temp,
+            File(temp, name="something.txt") as test_file,
+        ):
             self.assertFalse(test_file.writable())
 
     def test_seekable(self):
-        with tempfile.TemporaryFile() as temp, File(
-            temp, name="something.txt"
-        ) as test_file:
+        with (
+            tempfile.TemporaryFile() as temp,
+            File(temp, name="something.txt") as test_file,
+        ):
             self.assertTrue(test_file.seekable())
         self.assertFalse(test_file.seekable())
 
     def test_io_wrapper(self):
         content = "vive l'été\n"
-        with tempfile.TemporaryFile() as temp, File(
-            temp, name="something.txt"
-        ) as test_file:
+        with (
+            tempfile.TemporaryFile() as temp,
+            File(temp, name="something.txt") as test_file,
+        ):
             test_file.write(content.encode())
             test_file.seek(0)
             wrapper = TextIOWrapper(test_file, "utf-8", newline="\n")
@@ -200,6 +205,21 @@ class FileTests(unittest.TestCase):
             self.assertIs(locks.unlock(f1), True)
             self.assertIs(locks.unlock(f2), True)
 
+    def test_open_supports_full_signature(self):
+        called = False
+
+        def opener(path, flags):
+            nonlocal called
+            called = True
+            return os.open(path, flags)
+
+        file_path = Path(__file__).parent / "test.png"
+        with open(file_path) as f:
+            test_file = File(f)
+
+        with test_file.open(opener=opener):
+            self.assertIs(called, True)
+
 
 class NoNameFileTestCase(unittest.TestCase):
     """
@@ -419,35 +439,61 @@ class FileMoveSafeTests(unittest.TestCase):
         os.close(handle_a)
         os.close(handle_b)
 
-    def test_file_move_copystat_cifs(self):
+    def test_file_move_permissionerror(self):
         """
-        file_move_safe() ignores a copystat() EPERM PermissionError. This
-        happens when the destination filesystem is CIFS, for example.
+        file_move_safe() ignores PermissionError thrown by copystat() and
+        copymode().
+        For example, PermissionError can be raised when the destination
+        filesystem is CIFS, or when relabel is disabled by SELinux across
+        filesystems.
         """
-        copystat_EACCES_error = PermissionError(errno.EACCES, "msg")
-        copystat_EPERM_error = PermissionError(errno.EPERM, "msg")
+        permission_error = PermissionError(errno.EPERM, "msg")
+        os_error = OSError("msg")
         handle_a, self.file_a = tempfile.mkstemp()
         handle_b, self.file_b = tempfile.mkstemp()
+        handle_c, self.file_c = tempfile.mkstemp()
         try:
             # This exception is required to reach the copystat() call in
             # file_safe_move().
             with mock.patch("django.core.files.move.os.rename", side_effect=OSError()):
-                # An error besides EPERM isn't ignored.
+                # An error besides PermissionError isn't ignored.
                 with mock.patch(
-                    "django.core.files.move.copystat", side_effect=copystat_EACCES_error
+                    "django.core.files.move.copystat", side_effect=os_error
                 ):
-                    with self.assertRaises(PermissionError):
+                    with self.assertRaises(OSError):
                         file_move_safe(self.file_a, self.file_b, allow_overwrite=True)
-                # EPERM is ignored.
+                # When copystat() throws PermissionError, copymode() error besides
+                # PermissionError isn't ignored.
                 with mock.patch(
-                    "django.core.files.move.copystat", side_effect=copystat_EPERM_error
+                    "django.core.files.move.copystat", side_effect=permission_error
+                ):
+                    with mock.patch(
+                        "django.core.files.move.copymode", side_effect=os_error
+                    ):
+                        with self.assertRaises(OSError):
+                            file_move_safe(
+                                self.file_a, self.file_b, allow_overwrite=True
+                            )
+                # PermissionError raised by copystat() is ignored.
+                with mock.patch(
+                    "django.core.files.move.copystat", side_effect=permission_error
                 ):
                     self.assertIsNone(
                         file_move_safe(self.file_a, self.file_b, allow_overwrite=True)
                     )
+                    # PermissionError raised by copymode() is ignored too.
+                    with mock.patch(
+                        "django.core.files.move.copymode", side_effect=permission_error
+                    ):
+                        self.assertIsNone(
+                            file_move_safe(
+                                self.file_c, self.file_b, allow_overwrite=True
+                            )
+                        )
         finally:
             os.close(handle_a)
             os.close(handle_b)
+            os.close(handle_c)
 
 
 class SpooledTempTests(unittest.TestCase):
