@@ -3,6 +3,7 @@ from django.core.handlers.wsgi import WSGIHandler, WSGIRequest, get_script_name
 from django.core.signals import request_finished, request_started
 from django.db import close_old_connections, connection
 from django.test import (
+    AsyncRequestFactory,
     RequestFactory,
     SimpleTestCase,
     TransactionTestCase,
@@ -128,6 +129,19 @@ class TransactionsPerRequestTests(TransactionTestCase):
         finally:
             connection.settings_dict["ATOMIC_REQUESTS"] = old_atomic_requests
         self.assertContains(response, "False")
+        try:
+            connection.settings_dict["ATOMIC_REQUESTS"] = True
+            response = self.client.get("/not_in_transaction_using_none/")
+        finally:
+            connection.settings_dict["ATOMIC_REQUESTS"] = old_atomic_requests
+        self.assertContains(response, "False")
+        try:
+            connection.settings_dict["ATOMIC_REQUESTS"] = True
+            response = self.client.get("/not_in_transaction_using_text/")
+        finally:
+            connection.settings_dict["ATOMIC_REQUESTS"] = old_atomic_requests
+        # The non_atomic_requests decorator is used for an incorrect table.
+        self.assertContains(response, "True")
 
 
 @override_settings(ROOT_URLCONF="handlers.urls")
@@ -158,7 +172,7 @@ class SignalsTests(SimpleTestCase):
     def test_request_signals_streaming_response(self):
         response = self.client.get("/streaming/")
         self.assertEqual(self.signals, ["started"])
-        self.assertEqual(b"".join(response.streaming_content), b"streaming content")
+        self.assertEqual(b"".join(list(response)), b"streaming content")
         self.assertEqual(self.signals, ["started", "finished"])
 
 
@@ -230,11 +244,27 @@ class HandlerRequestTests(SimpleTestCase):
             ("/no_response_cbv/", "handlers.views.NoResponse.__call__"),
         )
         for url, view in tests:
-            with self.subTest(url=url), self.assertRaisesMessage(
-                ValueError, msg % view
+            with (
+                self.subTest(url=url),
+                self.assertRaisesMessage(ValueError, msg % view),
             ):
                 self.client.get(url)
 
+    def test_streaming(self):
+        response = self.client.get("/streaming/")
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(b"".join(list(response)), b"streaming content")
+
+    def test_async_streaming(self):
+        response = self.client.get("/async_streaming/")
+        self.assertEqual(response.status_code, 200)
+        msg = (
+            "StreamingHttpResponse must consume asynchronous iterators in order to "
+            "serve them synchronously. Use a synchronous iterator instead."
+        )
+        with self.assertWarnsMessage(Warning, msg):
+            self.assertEqual(b"".join(list(response)), b"streaming content")
+
 
 class ScriptNameTests(SimpleTestCase):
     def test_get_script_name(self):
@@ -299,3 +329,28 @@ class AsyncHandlerRequestTests(SimpleTestCase):
         )
         with self.assertRaisesMessage(ValueError, msg):
             await self.async_client.get("/unawaited/")
+
+    @override_settings(FORCE_SCRIPT_NAME="/FORCED_PREFIX/")
+    def test_force_script_name(self):
+        async_request_factory = AsyncRequestFactory()
+        request = async_request_factory.request(**{"path": "/somepath/"})
+        self.assertEqual(request.path, "/FORCED_PREFIX/somepath/")
+
+    async def test_sync_streaming(self):
+        response = await self.async_client.get("/streaming/")
+        self.assertEqual(response.status_code, 200)
+        msg = (
+            "StreamingHttpResponse must consume synchronous iterators in order to "
+            "serve them asynchronously. Use an asynchronous iterator instead."
+        )
+        with self.assertWarnsMessage(Warning, msg):
+            self.assertEqual(
+                b"".join([chunk async for chunk in response]), b"streaming content"
+            )
+
+    async def test_async_streaming(self):
+        response = await self.async_client.get("/async_streaming/")
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(
+            b"".join([chunk async for chunk in response]), b"streaming content"
+        )
