@@ -1,5 +1,6 @@
 from django.db import IntegrityError, connection, transaction
 from django.test import TestCase
+from django.utils.deprecation import RemovedInDjango60Warning
 
 from .models import (
     Bar,
@@ -185,8 +186,8 @@ class OneToOneTests(TestCase):
         bar.save()
         self.p1.delete()
 
-        self.assertEqual(Place.objects.all().count(), 1)
-        self.assertEqual(UndergroundBar.objects.all().count(), 1)
+        self.assertEqual(Place.objects.count(), 1)
+        self.assertEqual(UndergroundBar.objects.count(), 1)
 
     def test_create_models_m2m(self):
         """
@@ -524,6 +525,29 @@ class OneToOneTests(TestCase):
             Director._meta.base_manager_name = None
             Director._meta._expire_cache()
 
+    def test_create_reverse_o2o_error(self):
+        msg = "The following fields do not exist in this model: restaurant"
+        with self.assertRaisesMessage(ValueError, msg):
+            Place.objects.create(restaurant=self.r1)
+
+    def test_get_or_create_reverse_o2o_error(self):
+        msg = "The following fields do not exist in this model: restaurant"
+        r2 = Restaurant.objects.create(
+            place=self.p2, serves_hot_dogs=True, serves_pizza=False
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            Place.objects.get_or_create(name="nonexistent", defaults={"restaurant": r2})
+
+    def test_update_or_create_reverse_o2o_error(self):
+        msg = "The following fields do not exist in this model: restaurant"
+        r2 = Restaurant.objects.create(
+            place=self.p2, serves_hot_dogs=True, serves_pizza=False
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            Place.objects.update_or_create(
+                name="nonexistent", defaults={"restaurant": r2}
+            )
+
     def test_hasattr_related_object(self):
         # The exception raised on attribute access when a related object
         # doesn't exist should be an instance of a subclass of `AttributeError`
@@ -583,3 +607,23 @@ class OneToOneTests(TestCase):
         self.b1.place_id = self.p2.pk
         self.b1.save()
         self.assertEqual(self.b1.place, self.p2)
+
+    def test_get_prefetch_queryset_warning(self):
+        places = Place.objects.all()
+        msg = (
+            "get_prefetch_queryset() is deprecated. Use get_prefetch_querysets() "
+            "instead."
+        )
+        with self.assertWarnsMessage(RemovedInDjango60Warning, msg):
+            Place.bar.get_prefetch_queryset(places)
+
+    def test_get_prefetch_querysets_invalid_querysets_length(self):
+        places = Place.objects.all()
+        msg = (
+            "querysets argument of get_prefetch_querysets() should have a length of 1."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            Place.bar.get_prefetch_querysets(
+                instances=places,
+                querysets=[Bar.objects.all(), Bar.objects.all()],
+            )
