@@ -2,17 +2,18 @@
 Indirection layer for PostgreSQL-specific fields, so the tests don't fail when
 run with a backend other than PostgreSQL.
 """
+
 import enum
 
 from django.db import models
 
 try:
+    from django.contrib.postgres.fields import CICharField  # RemovedInDjango51Warning.
+    from django.contrib.postgres.fields import CIEmailField  # RemovedInDjango51Warning.
+    from django.contrib.postgres.fields import CITextField  # RemovedInDjango51Warning.
     from django.contrib.postgres.fields import (
         ArrayField,
         BigIntegerRangeField,
-        CICharField,
-        CIEmailField,
-        CITextField,
         DateRangeField,
         DateTimeRangeField,
         DecimalRangeField,
@@ -47,9 +48,9 @@ except ImportError:
 
     ArrayField = DummyArrayField
     BigIntegerRangeField = models.Field
-    CICharField = models.Field
-    CIEmailField = models.Field
-    CITextField = models.Field
+    CICharField = models.Field  # RemovedInDjango51Warning.
+    CIEmailField = models.Field  # RemovedInDjango51Warning.
+    CITextField = models.Field  # RemovedInDjango51Warning.
     DateRangeField = models.Field
     DateTimeRangeField = DummyContinuousRangeField
     DecimalRangeField = DummyContinuousRangeField
