@@ -1,9 +1,10 @@
-from django.db import connection
+from django.db import connection, transaction
 from django.db.models import (
     CharField,
     F,
     Func,
     IntegerField,
+    JSONField,
     OuterRef,
     Q,
     Subquery,
@@ -15,7 +16,7 @@ from django.db.models.functions import Cast, Concat, Substr
 from django.test import skipUnlessDBFeature
 from django.test.utils import Approximate, ignore_warnings
 from django.utils import timezone
-from django.utils.deprecation import RemovedInDjango50Warning
+from django.utils.deprecation import RemovedInDjango51Warning
 
 from . import PostgreSQLTestCase
 from .models import AggregateTestModel, HotelReservation, Room, StatTestModel
@@ -83,36 +84,35 @@ class TestGeneralAggregate(PostgreSQLTestCase):
             ]
         )
 
-    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_empty_result_set(self):
         AggregateTestModel.objects.all().delete()
         tests = [
-            (ArrayAgg("char_field"), []),
-            (ArrayAgg("integer_field"), []),
-            (ArrayAgg("boolean_field"), []),
-            (BitAnd("integer_field"), None),
-            (BitOr("integer_field"), None),
-            (BoolAnd("boolean_field"), None),
-            (BoolOr("boolean_field"), None),
-            (JSONBAgg("integer_field"), []),
-            (StringAgg("char_field", delimiter=";"), ""),
+            ArrayAgg("char_field"),
+            ArrayAgg("integer_field"),
+            ArrayAgg("boolean_field"),
+            BitAnd("integer_field"),
+            BitOr("integer_field"),
+            BoolAnd("boolean_field"),
+            BoolOr("boolean_field"),
+            JSONBAgg("integer_field"),
+            StringAgg("char_field", delimiter=";"),
         ]
         if connection.features.has_bit_xor:
-            tests.append((BitXor("integer_field"), None))
-        for aggregation, expected_result in tests:
+            tests.append(BitXor("integer_field"))
+        for aggregation in tests:
             with self.subTest(aggregation=aggregation):
                 # Empty result with non-execution optimization.
                 with self.assertNumQueries(0):
                     values = AggregateTestModel.objects.none().aggregate(
                         aggregation=aggregation,
                     )
-                    self.assertEqual(values, {"aggregation": expected_result})
+                    self.assertEqual(values, {"aggregation": None})
                 # Empty result when query must be executed.
                 with self.assertNumQueries(1):
                     values = AggregateTestModel.objects.aggregate(
                         aggregation=aggregation,
                     )
-                    self.assertEqual(values, {"aggregation": expected_result})
+                    self.assertEqual(values, {"aggregation": None})
 
     def test_default_argument(self):
         AggregateTestModel.objects.all().delete()
@@ -124,7 +124,12 @@ class TestGeneralAggregate(PostgreSQLTestCase):
             (BitOr("integer_field", default=0), 0),
             (BoolAnd("boolean_field", default=False), False),
             (BoolOr("boolean_field", default=False), False),
-            (JSONBAgg("integer_field", default=Value('["<empty>"]')), ["<empty>"]),
+            (JSONBAgg("integer_field", default=["<empty>"]), ["<empty>"]),
+            (
+                JSONBAgg("integer_field", default=Value(["<empty>"], JSONField())),
+                ["<empty>"],
+            ),
+            (StringAgg("char_field", delimiter=";", default="<empty>"), "<empty>"),
             (
                 StringAgg("char_field", delimiter=";", default=Value("<empty>")),
                 "<empty>",
@@ -141,64 +146,64 @@ class TestGeneralAggregate(PostgreSQLTestCase):
                     )
                     self.assertEqual(values, {"aggregation": expected_result})
                 # Empty result when query must be executed.
-                with self.assertNumQueries(1):
+                with transaction.atomic(), self.assertNumQueries(1):
                     values = AggregateTestModel.objects.aggregate(
                         aggregation=aggregation,
                     )
                     self.assertEqual(values, {"aggregation": expected_result})
 
-    def test_convert_value_deprecation(self):
+    @ignore_warnings(category=RemovedInDjango51Warning)
+    def test_jsonb_agg_default_str_value(self):
         AggregateTestModel.objects.all().delete()
         queryset = AggregateTestModel.objects.all()
-
-        with self.assertWarnsMessage(
-            RemovedInDjango50Warning, ArrayAgg.deprecation_msg
-        ):
-            queryset.aggregate(aggregation=ArrayAgg("boolean_field"))
-
-        with self.assertWarnsMessage(
-            RemovedInDjango50Warning, JSONBAgg.deprecation_msg
-        ):
-            queryset.aggregate(aggregation=JSONBAgg("integer_field"))
-
-        with self.assertWarnsMessage(
-            RemovedInDjango50Warning, StringAgg.deprecation_msg
-        ):
-            queryset.aggregate(aggregation=StringAgg("char_field", delimiter=";"))
-
-        # No warnings raised if default argument provided.
-        self.assertEqual(
-            queryset.aggregate(aggregation=ArrayAgg("boolean_field", default=None)),
-            {"aggregation": None},
-        )
-        self.assertEqual(
-            queryset.aggregate(aggregation=JSONBAgg("integer_field", default=None)),
-            {"aggregation": None},
-        )
         self.assertEqual(
             queryset.aggregate(
-                aggregation=StringAgg("char_field", delimiter=";", default=None),
+                aggregation=JSONBAgg("integer_field", default=Value("<empty>"))
             ),
-            {"aggregation": None},
+            {"aggregation": "<empty>"},
         )
-        self.assertEqual(
+
+    def test_jsonb_agg_default_str_value_deprecation(self):
+        queryset = AggregateTestModel.objects.all()
+        msg = (
+            "Passing a Value() with an output_field that isn't a JSONField as "
+            "JSONBAgg(default) is deprecated. Pass default=Value('<empty>', "
+            "output_field=JSONField()) instead."
+        )
+        with self.assertWarnsMessage(RemovedInDjango51Warning, msg):
             queryset.aggregate(
-                aggregation=ArrayAgg("boolean_field", default=Value([]))
+                aggregation=JSONBAgg("integer_field", default=Value("<empty>"))
+            )
+        with self.assertWarnsMessage(RemovedInDjango51Warning, msg):
+            queryset.none().aggregate(
+                aggregation=JSONBAgg("integer_field", default=Value("<empty>"))
             ),
-            {"aggregation": []},
-        )
+
+    @ignore_warnings(category=RemovedInDjango51Warning)
+    def test_jsonb_agg_default_encoded_json_string(self):
+        AggregateTestModel.objects.all().delete()
+        queryset = AggregateTestModel.objects.all()
         self.assertEqual(
             queryset.aggregate(
                 aggregation=JSONBAgg("integer_field", default=Value("[]"))
             ),
             {"aggregation": []},
         )
-        self.assertEqual(
-            queryset.aggregate(
-                aggregation=StringAgg("char_field", delimiter=";", default=Value("")),
-            ),
-            {"aggregation": ""},
+
+    def test_jsonb_agg_default_encoded_json_string_deprecation(self):
+        queryset = AggregateTestModel.objects.all()
+        msg = (
+            "Passing an encoded JSON string as JSONBAgg(default) is deprecated. Pass "
+            "default=[] instead."
         )
+        with self.assertWarnsMessage(RemovedInDjango51Warning, msg):
+            queryset.aggregate(
+                aggregation=JSONBAgg("integer_field", default=Value("[]"))
+            )
+        with self.assertWarnsMessage(RemovedInDjango51Warning, msg):
+            queryset.none().aggregate(
+                aggregation=JSONBAgg("integer_field", default=Value("[]"))
+            )
 
     def test_array_agg_charfield(self):
         values = AggregateTestModel.objects.aggregate(arrayagg=ArrayAgg("char_field"))
@@ -308,6 +313,49 @@ class TestGeneralAggregate(PostgreSQLTestCase):
         )
         self.assertCountEqual(qs.get(), [1, 2])
 
+    def test_array_agg_filter_index(self):
+        aggr1 = AggregateTestModel.objects.create(integer_field=1)
+        aggr2 = AggregateTestModel.objects.create(integer_field=2)
+        StatTestModel.objects.bulk_create(
+            [
+                StatTestModel(related_field=aggr1, int1=1, int2=0),
+                StatTestModel(related_field=aggr1, int1=2, int2=1),
+                StatTestModel(related_field=aggr2, int1=3, int2=0),
+                StatTestModel(related_field=aggr2, int1=4, int2=1),
+            ]
+        )
+        qs = (
+            AggregateTestModel.objects.filter(pk__in=[aggr1.pk, aggr2.pk])
+            .annotate(
+                array=ArrayAgg("stattestmodel__int1", filter=Q(stattestmodel__int2=0))
+            )
+            .annotate(array_value=F("array__0"))
+            .values_list("array_value", flat=True)
+        )
+        self.assertCountEqual(qs, [1, 3])
+
+    def test_array_agg_filter_slice(self):
+        aggr1 = AggregateTestModel.objects.create(integer_field=1)
+        aggr2 = AggregateTestModel.objects.create(integer_field=2)
+        StatTestModel.objects.bulk_create(
+            [
+                StatTestModel(related_field=aggr1, int1=1, int2=0),
+                StatTestModel(related_field=aggr1, int1=2, int2=1),
+                StatTestModel(related_field=aggr2, int1=3, int2=0),
+                StatTestModel(related_field=aggr2, int1=4, int2=1),
+                StatTestModel(related_field=aggr2, int1=5, int2=0),
+            ]
+        )
+        qs = (
+            AggregateTestModel.objects.filter(pk__in=[aggr1.pk, aggr2.pk])
+            .annotate(
+                array=ArrayAgg("stattestmodel__int1", filter=Q(stattestmodel__int2=0))
+            )
+            .annotate(array_value=F("array__1_2"))
+            .values_list("array_value", flat=True)
+        )
+        self.assertCountEqual(qs, [[], [5]])
+
     def test_bit_and_general(self):
         values = AggregateTestModel.objects.filter(integer_field__in=[0, 1]).aggregate(
             bitand=BitAnd("integer_field")
