@@ -6,6 +6,7 @@ test case that is capable of testing the capabilities of
 the serializers. This includes all valid data values, plus
 forward, backwards and self references.
 """
+
 import datetime
 import decimal
 import uuid
@@ -449,7 +450,7 @@ def serializerTest(self, format):
     # Create all the objects defined in the test data
     objects = []
     instance_count = {}
-    for (func, pk, klass, datum) in test_data:
+    for func, pk, klass, datum in test_data:
         with connection.constraint_checks_disabled():
             objects.extend(func[0](pk, klass, datum))
 
@@ -468,7 +469,7 @@ def serializerTest(self, format):
 
     # Assert that the deserialized data is the same
     # as the original source
-    for (func, pk, klass, datum) in test_data:
+    for func, pk, klass, datum in test_data:
         func[1](self, pk, klass, datum)
 
     # Assert that the number of objects deserialized is the
