@@ -19,6 +19,8 @@ testing against the contexts and templates produced by a view,
 rather than the HTML rendered to the end-user.
 
 """
+
+import copy
 import itertools
 import tempfile
 from unittest import mock
@@ -80,6 +82,25 @@ class ClientTest(TestCase):
         self.assertEqual(response.context["var"], "\xf2")
         self.assertEqual(response.templates[0].name, "GET Template")
 
+    def test_copy_response(self):
+        tests = ["/cbv_view/", "/get_view/"]
+        for url in tests:
+            with self.subTest(url=url):
+                response = self.client.get(url)
+                response_copy = copy.copy(response)
+                self.assertEqual(repr(response), repr(response_copy))
+                self.assertIs(response_copy.client, response.client)
+                self.assertIs(response_copy.resolver_match, response.resolver_match)
+                self.assertIs(response_copy.wsgi_request, response.wsgi_request)
+
+    async def test_copy_response_async(self):
+        response = await self.async_client.get("/async_get_view/")
+        response_copy = copy.copy(response)
+        self.assertEqual(repr(response), repr(response_copy))
+        self.assertIs(response_copy.client, response.client)
+        self.assertIs(response_copy.resolver_match, response.resolver_match)
+        self.assertIs(response_copy.asgi_request, response.asgi_request)
+
     def test_query_string_encoding(self):
         # WSGI requires latin-1 encoded strings.
         response = self.client.get("/get_view/?var=1\ufffd")
@@ -1050,6 +1071,56 @@ class RequestFactoryTest(SimpleTestCase):
         echoed_request_line = "TRACE {} {}".format(url_path, protocol)
         self.assertContains(response, echoed_request_line)
 
+    def test_request_factory_default_headers(self):
+        request = RequestFactory(
+            headers={
+                "authorization": "Bearer faketoken",
+                "x-another-header": "some other value",
+            }
+        ).get("/somewhere/")
+        self.assertEqual(request.headers["authorization"], "Bearer faketoken")
+        self.assertIn("HTTP_AUTHORIZATION", request.META)
+        self.assertEqual(request.headers["x-another-header"], "some other value")
+        self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
+
+        request = RequestFactory(
+            headers={
+                "Authorization": "Bearer faketoken",
+                "X-Another-Header": "some other value",
+            }
+        ).get("/somewhere/")
+        self.assertEqual(request.headers["authorization"], "Bearer faketoken")
+        self.assertIn("HTTP_AUTHORIZATION", request.META)
+        self.assertEqual(request.headers["x-another-header"], "some other value")
+        self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
+
+    def test_request_factory_sets_headers(self):
+        for method_name, view in self.http_methods_and_views:
+            method = getattr(self.request_factory, method_name)
+            request = method(
+                "/somewhere/",
+                headers={
+                    "authorization": "Bearer faketoken",
+                    "x-another-header": "some other value",
+                },
+            )
+            self.assertEqual(request.headers["authorization"], "Bearer faketoken")
+            self.assertIn("HTTP_AUTHORIZATION", request.META)
+            self.assertEqual(request.headers["x-another-header"], "some other value")
+            self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
+
+            request = method(
+                "/somewhere/",
+                headers={
+                    "Authorization": "Bearer faketoken",
+                    "X-Another-Header": "some other value",
+                },
+            )
+            self.assertEqual(request.headers["authorization"], "Bearer faketoken")
+            self.assertIn("HTTP_AUTHORIZATION", request.META)
+            self.assertEqual(request.headers["x-another-header"], "some other value")
+            self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
+
 
 @override_settings(ROOT_URLCONF="test_client.urls")
 class AsyncClientTest(TestCase):
@@ -1065,8 +1136,11 @@ class AsyncClientTest(TestCase):
         response = await self.async_client.get("/middleware_urlconf_view/")
         self.assertEqual(response.resolver_match.url_name, "middleware_urlconf_view")
 
-    async def test_follow_parameter_not_implemented(self):
-        msg = "AsyncClient request methods do not accept the follow parameter."
+    async def test_redirect(self):
+        response = await self.async_client.get("/redirect_view/")
+        self.assertEqual(response.status_code, 302)
+
+    async def test_follow_redirect(self):
         tests = (
             "get",
             "post",
@@ -1080,13 +1154,29 @@ class AsyncClientTest(TestCase):
         for method_name in tests:
             with self.subTest(method=method_name):
                 method = getattr(self.async_client, method_name)
-                with self.assertRaisesMessage(NotImplementedError, msg):
-                    await method("/redirect_view/", follow=True)
+                response = await method("/redirect_view/", follow=True)
+                self.assertEqual(response.status_code, 200)
+                self.assertEqual(response.resolver_match.url_name, "get_view")
+
+    async def test_follow_double_redirect(self):
+        response = await self.async_client.get("/double_redirect_view/", follow=True)
+        self.assertRedirects(
+            response, "/get_view/", status_code=302, target_status_code=200
+        )
+        self.assertEqual(len(response.redirect_chain), 2)
 
     async def test_get_data(self):
         response = await self.async_client.get("/get_view/", {"var": "val"})
         self.assertContains(response, "This is a test. val is the value.")
 
+    async def test_post_data(self):
+        response = await self.async_client.post("/post_view/", {"value": 37})
+        self.assertContains(response, "Data received: 37 is the value.")
+
+    async def test_body_read_on_get_data(self):
+        response = await self.async_client.get("/post_view/")
+        self.assertContains(response, "Viewing GET page.")
+
 
 @override_settings(ROOT_URLCONF="test_client.urls")
 class AsyncRequestFactoryTest(SimpleTestCase):
@@ -1131,6 +1221,16 @@ class AsyncRequestFactoryTest(SimpleTestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content, b'{"example": "data"}')
 
+    async def test_request_limited_read(self):
+        tests = ["GET", "POST"]
+        for method in tests:
+            with self.subTest(method=method):
+                request = self.request_factory.generic(
+                    method,
+                    "/somewhere",
+                )
+                self.assertEqual(request.read(200), b"")
+
     def test_request_factory_sets_headers(self):
         request = self.request_factory.get(
             "/somewhere/",
@@ -1142,6 +1242,18 @@ class AsyncRequestFactoryTest(SimpleTestCase):
         self.assertEqual(request.headers["x-another-header"], "some other value")
         self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
 
+        request = self.request_factory.get(
+            "/somewhere/",
+            headers={
+                "Authorization": "Bearer faketoken",
+                "X-Another-Header": "some other value",
+            },
+        )
+        self.assertEqual(request.headers["authorization"], "Bearer faketoken")
+        self.assertIn("HTTP_AUTHORIZATION", request.META)
+        self.assertEqual(request.headers["x-another-header"], "some other value")
+        self.assertIn("HTTP_X_ANOTHER_HEADER", request.META)
+
     def test_request_factory_query_string(self):
         request = self.request_factory.get("/somewhere/", {"example": "data"})
         self.assertNotIn("Query-String", request.headers)
