@@ -1,11 +1,11 @@
-import logging
 import os
+import sys
 import unittest
 import warnings
 from io import StringIO
 from unittest import mock
 
-from django.conf import settings
+from django.conf import STATICFILES_STORAGE_ALIAS, settings
 from django.contrib.staticfiles.finders import get_finder, get_finders
 from django.contrib.staticfiles.storage import staticfiles_storage
 from django.core.exceptions import ImproperlyConfigured
@@ -27,6 +27,7 @@ from django.forms import (
     formset_factory,
 )
 from django.http import HttpResponse
+from django.template import Context, Template
 from django.template.loader import render_to_string
 from django.test import (
     SimpleTestCase,
@@ -46,8 +47,8 @@ from django.test.utils import (
     setup_test_environment,
 )
 from django.urls import NoReverseMatch, path, reverse, reverse_lazy
-from django.utils.deprecation import RemovedInDjango50Warning
-from django.utils.log import DEFAULT_LOGGING
+from django.utils.deprecation import RemovedInDjango51Warning
+from django.utils.html import VOID_ELEMENTS
 from django.utils.version import PY311
 
 from .models import Car, Person, PossessedCar
@@ -182,7 +183,11 @@ class SkippingClassTestCase(TestCase):
         except unittest.SkipTest:
             self.fail("SkipTest should not be raised here.")
         result = unittest.TextTestRunner(stream=StringIO()).run(test_suite)
-        self.assertEqual(result.testsRun, 3)
+        # PY312: Python 3.12.1 does not include skipped tests in the number of
+        # running tests.
+        self.assertEqual(
+            result.testsRun, 1 if sys.version_info[:3] == (3, 12, 1) else 3
+        )
         self.assertEqual(len(result.skipped), 2)
         self.assertEqual(result.skipped[0][1], "Database has feature(s) __class__")
         self.assertEqual(result.skipped[1][1], "Database has feature(s) __class__")
@@ -236,10 +241,6 @@ class AssertNumQueriesTests(TestCase):
         self.assertNumQueries(2, test_func)
 
 
-@unittest.skipUnless(
-    connection.vendor != "sqlite" or not connection.is_in_memory_db(),
-    "For SQLite in-memory tests, closing the connection destroys the database.",
-)
 class AssertNumQueriesUponConnectionTests(TransactionTestCase):
     available_apps = []
 
@@ -265,47 +266,56 @@ class AssertNumQueriesUponConnectionTests(TransactionTestCase):
                 list(Car.objects.all())
 
 
-class AssertQuerysetEqualTests(TestCase):
+class AssertQuerySetEqualTests(TestCase):
     @classmethod
     def setUpTestData(cls):
         cls.p1 = Person.objects.create(name="p1")
         cls.p2 = Person.objects.create(name="p2")
 
-    def test_empty(self):
+    def test_rename_assertquerysetequal_deprecation_warning(self):
+        msg = "assertQuerysetEqual() is deprecated in favor of assertQuerySetEqual()."
+        with self.assertRaisesMessage(RemovedInDjango51Warning, msg):
+            self.assertQuerysetEqual()
+
+    @ignore_warnings(category=RemovedInDjango51Warning)
+    def test_deprecated_assertquerysetequal(self):
         self.assertQuerysetEqual(Person.objects.filter(name="p3"), [])
 
+    def test_empty(self):
+        self.assertQuerySetEqual(Person.objects.filter(name="p3"), [])
+
     def test_ordered(self):
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Person.objects.order_by("name"),
             [self.p1, self.p2],
         )
 
     def test_unordered(self):
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Person.objects.order_by("name"), [self.p2, self.p1], ordered=False
         )
 
     def test_queryset(self):
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Person.objects.order_by("name"),
             Person.objects.order_by("name"),
         )
 
     def test_flat_values_list(self):
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Person.objects.order_by("name").values_list("name", flat=True),
             ["p1", "p2"],
         )
 
     def test_transform(self):
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Person.objects.order_by("name"),
             [self.p1.pk, self.p2.pk],
             transform=lambda x: x.pk,
         )
 
     def test_repr_transform(self):
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Person.objects.order_by("name"),
             [repr(self.p1), repr(self.p2)],
             transform=repr,
@@ -319,16 +329,16 @@ class AssertQuerysetEqualTests(TestCase):
             "ordered value."
         )
         with self.assertRaisesMessage(ValueError, msg):
-            self.assertQuerysetEqual(
+            self.assertQuerySetEqual(
                 Person.objects.all(),
                 [self.p1, self.p2],
             )
         # No error for one value.
-        self.assertQuerysetEqual(Person.objects.filter(name="p1"), [self.p1])
+        self.assertQuerySetEqual(Person.objects.filter(name="p1"), [self.p1])
 
     def test_repeated_values(self):
         """
-        assertQuerysetEqual checks the number of appearance of each item
+        assertQuerySetEqual checks the number of appearance of each item
         when used with option ordered=False.
         """
         batmobile = Car.objects.create(name="Batmobile")
@@ -344,10 +354,10 @@ class AssertQuerysetEqualTests(TestCase):
             ]
         )
         with self.assertRaises(AssertionError):
-            self.assertQuerysetEqual(
+            self.assertQuerySetEqual(
                 self.p1.cars.all(), [batmobile, k2000], ordered=False
             )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             self.p1.cars.all(), [batmobile] * 2 + [k2000] * 4, ordered=False
         )
 
@@ -357,7 +367,7 @@ class AssertQuerysetEqualTests(TestCase):
         names.append("Extra Person")
 
         with self.assertRaises(AssertionError) as ctx:
-            self.assertQuerysetEqual(
+            self.assertQuerySetEqual(
                 Person.objects.filter(name__startswith="Joe"),
                 names,
                 ordered=False,
@@ -369,7 +379,7 @@ class AssertQuerysetEqualTests(TestCase):
         self.maxDiff = None
         try:
             with self.assertRaises(AssertionError) as ctx:
-                self.assertQuerysetEqual(
+                self.assertQuerySetEqual(
                     Person.objects.filter(name__startswith="Joe"),
                     names,
                     ordered=False,
@@ -537,12 +547,20 @@ class AssertTemplateUsedContextManagerTests(SimpleTestCase):
             with self.assertTemplateUsed("template_used/base.html"):
                 render_to_string("template_used/alternative.html")
 
-        with self.assertRaisesMessage(
-            AssertionError, "No templates used to render the response"
-        ):
+        msg = "No templates used to render the response"
+        with self.assertRaisesMessage(AssertionError, msg):
             response = self.client.get("/test_utils/no_template_used/")
             self.assertTemplateUsed(response, "template_used/base.html")
 
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertTemplateUsed("template_used/base.html"):
+                self.client.get("/test_utils/no_template_used/")
+
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertTemplateUsed("template_used/base.html"):
+                template = Template("template_used/alternative.html", name=None)
+                template.render(Context())
+
     def test_msg_prefix(self):
         msg_prefix = "Prefix"
         msg = f"{msg_prefix}: No templates used to render the response"
@@ -654,27 +672,8 @@ class HTMLEqualTests(SimpleTestCase):
         self.assertEqual(len(dom.children), 1)
         self.assertEqual(dom.children[0], "<p>foo</p> '</scr'+'ipt>' <span>bar</span>")
 
-    def test_self_closing_tags(self):
-        self_closing_tags = [
-            "area",
-            "base",
-            "br",
-            "col",
-            "embed",
-            "hr",
-            "img",
-            "input",
-            "link",
-            "meta",
-            "param",
-            "source",
-            "track",
-            "wbr",
-            # Deprecated tags
-            "frame",
-            "spacer",
-        ]
-        for tag in self_closing_tags:
+    def test_void_elements(self):
+        for tag in VOID_ELEMENTS:
             with self.subTest(tag):
                 dom = parse_html("<p>Hello <%s> world</p>" % tag)
                 self.assertEqual(len(dom.children), 3)
@@ -1194,47 +1193,6 @@ class AssertWarnsMessageTests(SimpleTestCase):
             func1()
 
 
-# TODO: Remove when dropping support for PY39.
-class AssertNoLogsTest(SimpleTestCase):
-    @classmethod
-    def setUpClass(cls):
-        super().setUpClass()
-        logging.config.dictConfig(DEFAULT_LOGGING)
-        cls.addClassCleanup(logging.config.dictConfig, settings.LOGGING)
-
-    def setUp(self):
-        self.logger = logging.getLogger("django")
-
-    @override_settings(DEBUG=True)
-    def test_fails_when_log_emitted(self):
-        msg = "Unexpected logs found: ['INFO:django:FAIL!']"
-        with self.assertRaisesMessage(AssertionError, msg):
-            with self.assertNoLogs("django", "INFO"):
-                self.logger.info("FAIL!")
-
-    @override_settings(DEBUG=True)
-    def test_text_level(self):
-        with self.assertNoLogs("django", "INFO"):
-            self.logger.debug("DEBUG logs are ignored.")
-
-    @override_settings(DEBUG=True)
-    def test_int_level(self):
-        with self.assertNoLogs("django", logging.INFO):
-            self.logger.debug("DEBUG logs are ignored.")
-
-    @override_settings(DEBUG=True)
-    def test_default_level(self):
-        with self.assertNoLogs("django"):
-            self.logger.debug("DEBUG logs are ignored.")
-
-    @override_settings(DEBUG=True)
-    def test_does_not_hide_other_failures(self):
-        msg = "1 != 2"
-        with self.assertRaisesMessage(AssertionError, msg):
-            with self.assertNoLogs("django"):
-                self.assertEqual(1, 2)
-
-
 class AssertFieldOutputTests(SimpleTestCase):
     def test_assert_field_output(self):
         error_invalid = ["Enter a valid email address."]
@@ -1379,44 +1337,6 @@ class TestFormset(formset_factory(TestForm)):
 
 
 class AssertFormErrorTests(SimpleTestCase):
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_non_client_response(self):
-        msg = (
-            "assertFormError() is only usable on responses fetched using the "
-            "Django test Client."
-        )
-        response = HttpResponse()
-        with self.assertRaisesMessage(ValueError, msg):
-            self.assertFormError(response, "form", "field", "invalid value")
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_response_with_no_context(self):
-        msg = "Response did not use any contexts to render the response"
-        response = mock.Mock(context=[])
-        with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormError(response, "form", "field", "invalid value")
-        msg_prefix = "Custom prefix"
-        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormError(
-                response,
-                "form",
-                "field",
-                "invalid value",
-                msg_prefix=msg_prefix,
-            )
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_form_not_in_context(self):
-        msg = "The form 'form' was not used to render the response"
-        response = mock.Mock(context=[{}])
-        with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormError(response, "form", "field", "invalid value")
-        msg_prefix = "Custom prefix"
-        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormError(
-                response, "form", "field", "invalid value", msg_prefix=msg_prefix
-            )
-
     def test_single_error(self):
         self.assertFormError(TestForm.invalid(), "field", "invalid value")
 
@@ -1518,44 +1438,24 @@ class AssertFormErrorTests(SimpleTestCase):
             )
 
 
-class AssertFormsetErrorTests(SimpleTestCase):
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_non_client_response(self):
-        msg = (
-            "assertFormsetError() is only usable on responses fetched using "
-            "the Django test Client."
-        )
-        response = HttpResponse()
-        with self.assertRaisesMessage(ValueError, msg):
-            self.assertFormsetError(response, "formset", 0, "field", "invalid value")
+class AssertFormSetErrorTests(SimpleTestCase):
+    def test_rename_assertformseterror_deprecation_warning(self):
+        msg = "assertFormsetError() is deprecated in favor of assertFormSetError()."
+        with self.assertRaisesMessage(RemovedInDjango51Warning, msg):
+            self.assertFormsetError()
 
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_response_with_no_context(self):
-        msg = "Response did not use any contexts to render the response"
-        response = mock.Mock(context=[])
-        with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(response, "formset", 0, "field", "invalid value")
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_formset_not_in_context(self):
-        msg = "The formset 'formset' was not used to render the response"
-        response = mock.Mock(context=[{}])
-        with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(response, "formset", 0, "field", "invalid value")
-        msg_prefix = "Custom prefix"
-        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
-                response, "formset", 0, "field", "invalid value", msg_prefix=msg_prefix
-            )
-
-    def test_single_error(self):
+    @ignore_warnings(category=RemovedInDjango51Warning)
+    def test_deprecated_assertformseterror(self):
         self.assertFormsetError(TestFormset.invalid(), 0, "field", "invalid value")
 
+    def test_single_error(self):
+        self.assertFormSetError(TestFormset.invalid(), 0, "field", "invalid value")
+
     def test_error_list(self):
-        self.assertFormsetError(TestFormset.invalid(), 0, "field", ["invalid value"])
+        self.assertFormSetError(TestFormset.invalid(), 0, "field", ["invalid value"])
 
     def test_empty_errors_valid_formset(self):
-        self.assertFormsetError(TestFormset.valid(), 0, "field", [])
+        self.assertFormSetError(TestFormset.valid(), 0, "field", [])
 
     def test_multiple_forms(self):
         formset = TestFormset(
@@ -1567,8 +1467,8 @@ class AssertFormsetErrorTests(SimpleTestCase):
             }
         )
         formset.full_clean()
-        self.assertFormsetError(formset, 0, "field", [])
-        self.assertFormsetError(formset, 1, "field", ["invalid value"])
+        self.assertFormSetError(formset, 0, "field", [])
+        self.assertFormSetError(formset, 1, "field", ["invalid value"])
 
     def test_field_not_in_form(self):
         msg = (
@@ -1576,12 +1476,12 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "does not contain the field 'other_field'."
         )
         with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(), 0, "other_field", "invalid value"
             )
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(),
                 0,
                 "other_field",
@@ -1595,11 +1495,11 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "valid=True total_forms=1> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(TestFormset.valid(), 0, "field", "invalid value")
+            self.assertFormSetError(TestFormset.valid(), 0, "field", "invalid value")
         self.assertIn("[] != ['invalid value']", str(ctx.exception))
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.valid(), 0, "field", "invalid value", msg_prefix=msg_prefix
             )
 
@@ -1609,11 +1509,11 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "valid=False total_forms=1> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(TestFormset.invalid(), 0, "field", "other error")
+            self.assertFormSetError(TestFormset.invalid(), 0, "field", "other error")
         self.assertIn("['invalid value'] != ['other error']", str(ctx.exception))
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(), 0, "field", "other error", msg_prefix=msg_prefix
             )
 
@@ -1623,7 +1523,7 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "bound, it will never have any errors."
         )
         with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(TestFormset(), 0, "field", [])
+            self.assertFormSetError(TestFormset(), 0, "field", [])
 
     def test_empty_errors_invalid_formset(self):
         msg = (
@@ -1631,11 +1531,11 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "valid=False total_forms=1> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(TestFormset.invalid(), 0, "field", [])
+            self.assertFormSetError(TestFormset.invalid(), 0, "field", [])
         self.assertIn("['invalid value'] != []", str(ctx.exception))
 
     def test_non_field_errors(self):
-        self.assertFormsetError(
+        self.assertFormSetError(
             TestFormset.invalid(nonfield=True), 0, None, "non-field error"
         )
 
@@ -1645,7 +1545,7 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "valid=False total_forms=1> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(nonfield=True), 0, None, "other non-field error"
             )
         self.assertIn(
@@ -1653,7 +1553,7 @@ class AssertFormsetErrorTests(SimpleTestCase):
         )
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(nonfield=True),
                 0,
                 None,
@@ -1667,16 +1567,16 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "valid=False total_forms=1> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(TestFormset.invalid(), 0, None, "non-field error")
+            self.assertFormSetError(TestFormset.invalid(), 0, None, "non-field error")
         self.assertIn("[] != ['non-field error']", str(ctx.exception))
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(), 0, None, "non-field error", msg_prefix=msg_prefix
             )
 
     def test_non_form_errors(self):
-        self.assertFormsetError(TestFormset.invalid(nonform=True), None, None, "error")
+        self.assertFormSetError(TestFormset.invalid(nonform=True), None, None, "error")
 
     def test_different_non_form_errors(self):
         msg = (
@@ -1684,13 +1584,13 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "total_forms=0> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(nonform=True), None, None, "other error"
             )
         self.assertIn("['error'] != ['other error']", str(ctx.exception))
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(nonform=True),
                 None,
                 None,
@@ -1704,11 +1604,11 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "total_forms=1> don't match."
         )
         with self.assertRaisesMessage(AssertionError, msg) as ctx:
-            self.assertFormsetError(TestFormset.invalid(), None, None, "error")
+            self.assertFormSetError(TestFormset.invalid(), None, None, "error")
         self.assertIn("[] != ['error']", str(ctx.exception))
         msg_prefix = "Custom prefix"
         with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(),
                 None,
                 None,
@@ -1719,7 +1619,7 @@ class AssertFormsetErrorTests(SimpleTestCase):
     def test_non_form_errors_with_field(self):
         msg = "You must use field=None with form_index=None."
         with self.assertRaisesMessage(ValueError, msg):
-            self.assertFormsetError(
+            self.assertFormSetError(
                 TestFormset.invalid(nonform=True), None, "field", "error"
             )
 
@@ -1729,7 +1629,7 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "1 form."
         )
         with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(TestFormset.invalid(), 2, "field", "error")
+            self.assertFormSetError(TestFormset.invalid(), 2, "field", "error")
 
     def test_form_index_too_big_plural(self):
         formset = TestFormset(
@@ -1746,220 +1646,7 @@ class AssertFormsetErrorTests(SimpleTestCase):
             "forms."
         )
         with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(formset, 2, "field", "error")
-
-
-# RemovedInDjango50Warning
-class AssertFormErrorDeprecationTests(SimpleTestCase):
-    """
-    Exhaustively test all possible combinations of args/kwargs for the old
-    signature.
-    """
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_assert_form_error_errors_none(self):
-        msg = (
-            "The errors of field 'field' on form <TestForm bound=True, valid=False, "
-            "fields=(field)> don't match."
-        )
-        with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormError(TestForm.invalid(), "field", None)
-
-    def test_assert_form_error_errors_none_warning(self):
-        msg = (
-            "Passing errors=None to assertFormError() is deprecated, use "
-            "errors=[] instead."
-        )
-        with self.assertWarnsMessage(RemovedInDjango50Warning, msg):
-            self.assertFormError(TestForm.valid(), "field", None)
-
-    def _assert_form_error_old_api_cases(self, form, field, errors, msg_prefix):
-        response = mock.Mock(context=[{"form": TestForm.invalid()}])
-        return (
-            ((response, form, field, errors), {}),
-            ((response, form, field, errors, msg_prefix), {}),
-            ((response, form, field, errors), {"msg_prefix": msg_prefix}),
-            ((response, form, field), {"errors": errors}),
-            ((response, form, field), {"errors": errors, "msg_prefix": msg_prefix}),
-            ((response, form), {"field": field, "errors": errors}),
-            (
-                (response, form),
-                {"field": field, "errors": errors, "msg_prefix": msg_prefix},
-            ),
-            ((response,), {"form": form, "field": field, "errors": errors}),
-            (
-                (response,),
-                {
-                    "form": form,
-                    "field": field,
-                    "errors": errors,
-                    "msg_prefix": msg_prefix,
-                },
-            ),
-            (
-                (),
-                {"response": response, "form": form, "field": field, "errors": errors},
-            ),
-            (
-                (),
-                {
-                    "response": response,
-                    "form": form,
-                    "field": field,
-                    "errors": errors,
-                    "msg_prefix": msg_prefix,
-                },
-            ),
-        )
-
-    def test_assert_form_error_old_api(self):
-        deprecation_msg = (
-            "Passing response to assertFormError() is deprecated. Use the form object "
-            "directly: assertFormError(response.context['form'], 'field', ...)"
-        )
-        for args, kwargs in self._assert_form_error_old_api_cases(
-            form="form",
-            field="field",
-            errors=["invalid value"],
-            msg_prefix="Custom prefix",
-        ):
-            with self.subTest(args=args, kwargs=kwargs):
-                with self.assertWarnsMessage(RemovedInDjango50Warning, deprecation_msg):
-                    self.assertFormError(*args, **kwargs)
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_assert_form_error_old_api_assertion_error(self):
-        for args, kwargs in self._assert_form_error_old_api_cases(
-            form="form",
-            field="field",
-            errors=["other error"],
-            msg_prefix="Custom prefix",
-        ):
-            with self.subTest(args=args, kwargs=kwargs):
-                with self.assertRaises(AssertionError):
-                    self.assertFormError(*args, **kwargs)
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_assert_formset_error_errors_none(self):
-        msg = (
-            "The errors of field 'field' on form 0 of formset <TestFormset: bound=True "
-            "valid=False total_forms=1> don't match."
-        )
-        with self.assertRaisesMessage(AssertionError, msg):
-            self.assertFormsetError(TestFormset.invalid(), 0, "field", None)
-
-    def test_assert_formset_error_errors_none_warning(self):
-        msg = (
-            "Passing errors=None to assertFormsetError() is deprecated, use "
-            "errors=[] instead."
-        )
-        with self.assertWarnsMessage(RemovedInDjango50Warning, msg):
-            self.assertFormsetError(TestFormset.valid(), 0, "field", None)
-
-    def _assert_formset_error_old_api_cases(
-        self, formset, form_index, field, errors, msg_prefix
-    ):
-        response = mock.Mock(context=[{"formset": TestFormset.invalid()}])
-        return (
-            ((response, formset, form_index, field, errors), {}),
-            ((response, formset, form_index, field, errors, msg_prefix), {}),
-            (
-                (response, formset, form_index, field, errors),
-                {"msg_prefix": msg_prefix},
-            ),
-            ((response, formset, form_index, field), {"errors": errors}),
-            (
-                (response, formset, form_index, field),
-                {"errors": errors, "msg_prefix": msg_prefix},
-            ),
-            ((response, formset, form_index), {"field": field, "errors": errors}),
-            (
-                (response, formset, form_index),
-                {"field": field, "errors": errors, "msg_prefix": msg_prefix},
-            ),
-            (
-                (response, formset),
-                {"form_index": form_index, "field": field, "errors": errors},
-            ),
-            (
-                (response, formset),
-                {
-                    "form_index": form_index,
-                    "field": field,
-                    "errors": errors,
-                    "msg_prefix": msg_prefix,
-                },
-            ),
-            (
-                (response,),
-                {
-                    "formset": formset,
-                    "form_index": form_index,
-                    "field": field,
-                    "errors": errors,
-                },
-            ),
-            (
-                (response,),
-                {
-                    "formset": formset,
-                    "form_index": form_index,
-                    "field": field,
-                    "errors": errors,
-                    "msg_prefix": msg_prefix,
-                },
-            ),
-            (
-                (),
-                {
-                    "response": response,
-                    "formset": formset,
-                    "form_index": form_index,
-                    "field": field,
-                    "errors": errors,
-                },
-            ),
-            (
-                (),
-                {
-                    "response": response,
-                    "formset": formset,
-                    "form_index": form_index,
-                    "field": field,
-                    "errors": errors,
-                    "msg_prefix": msg_prefix,
-                },
-            ),
-        )
-
-    def test_assert_formset_error_old_api(self):
-        deprecation_msg = (
-            "Passing response to assertFormsetError() is deprecated. Use the formset "
-            "object directly: assertFormsetError(response.context['formset'], 0, ...)"
-        )
-        for args, kwargs in self._assert_formset_error_old_api_cases(
-            formset="formset",
-            form_index=0,
-            field="field",
-            errors=["invalid value"],
-            msg_prefix="Custom prefix",
-        ):
-            with self.subTest(args=args, kwargs=kwargs):
-                with self.assertWarnsMessage(RemovedInDjango50Warning, deprecation_msg):
-                    self.assertFormsetError(*args, **kwargs)
-
-    @ignore_warnings(category=RemovedInDjango50Warning)
-    def test_assert_formset_error_old_api_assertion_error(self):
-        for args, kwargs in self._assert_formset_error_old_api_cases(
-            formset="formset",
-            form_index=0,
-            field="field",
-            errors=["other error"],
-            msg_prefix="Custom prefix",
-        ):
-            with self.subTest(args=args, kwargs=kwargs):
-                with self.assertRaises(AssertionError):
-                    self.assertFormsetError(*args, **kwargs)
+            self.assertFormSetError(formset, 2, "field", "error")
 
 
 class FirstUrls:
@@ -2092,12 +1779,14 @@ class OverrideSettingsTests(SimpleTestCase):
 
     def test_override_staticfiles_storage(self):
         """
-        Overriding the STATICFILES_STORAGE setting should be reflected in
+        Overriding the STORAGES setting should be reflected in
         the value of django.contrib.staticfiles.storage.staticfiles_storage.
         """
         new_class = "ManifestStaticFilesStorage"
         new_storage = "django.contrib.staticfiles.storage." + new_class
-        with self.settings(STATICFILES_STORAGE=new_storage):
+        with self.settings(
+            STORAGES={STATICFILES_STORAGE_ALIAS: {"BACKEND": new_storage}}
+        ):
             self.assertEqual(staticfiles_storage.__class__.__name__, new_class)
 
     def test_override_staticfiles_finders(self):
@@ -2285,6 +1974,32 @@ class CaptureOnCommitCallbacksTests(TestCase):
 
         self.assertEqual(callbacks, [branch_1, branch_2, leaf_3, leaf_1, leaf_2])
 
+    def test_execute_robust(self):
+        class MyException(Exception):
+            pass
+
+        def hook():
+            self.callback_called = True
+            raise MyException("robust callback")
+
+        with self.assertLogs("django.test", "ERROR") as cm:
+            with self.captureOnCommitCallbacks(execute=True) as callbacks:
+                transaction.on_commit(hook, robust=True)
+
+        self.assertEqual(len(callbacks), 1)
+        self.assertIs(self.callback_called, True)
+
+        log_record = cm.records[0]
+        self.assertEqual(
+            log_record.getMessage(),
+            "Error calling CaptureOnCommitCallbacksTests.test_execute_robust.<locals>."
+            "hook in on_commit() (robust callback).",
+        )
+        self.assertIsNotNone(log_record.exc_info)
+        raised_exception = log_record.exc_info[1]
+        self.assertIsInstance(raised_exception, MyException)
+        self.assertEqual(str(raised_exception), "robust callback")
+
 
 class DisallowedDatabaseQueriesTests(SimpleTestCase):
     def test_disallowed_database_connections(self):
