@@ -9,16 +9,12 @@ import time
 import types
 import weakref
 import zipfile
+import zoneinfo
 from importlib import import_module
 from pathlib import Path
 from subprocess import CompletedProcess
 from unittest import mock, skip, skipIf
 
-try:
-    import zoneinfo
-except ImportError:
-    from backports import zoneinfo
-
 import django.__main__
 from django.apps.registry import Apps
 from django.test import SimpleTestCase
@@ -242,6 +238,17 @@ class TestChildArguments(SimpleTestCase):
                     autoreload.get_child_arguments(), [exe_path, "runserver"]
                 )
 
+    @mock.patch("sys.warnoptions", [])
+    @mock.patch.dict(sys.modules, {"__main__": django.__main__})
+    def test_use_exe_when_main_spec(self):
+        with tempfile.TemporaryDirectory() as tmpdir:
+            exe_path = Path(tmpdir) / "django-admin.exe"
+            exe_path.touch()
+            with mock.patch("sys.argv", [exe_path.with_suffix(""), "runserver"]):
+                self.assertEqual(
+                    autoreload.get_child_arguments(), [exe_path, "runserver"]
+                )
+
     @mock.patch("__main__.__spec__", None)
     @mock.patch("sys.warnoptions", [])
     @mock.patch("sys._xoptions", {})
@@ -381,16 +388,6 @@ class RunWithReloaderTests(SimpleTestCase):
 
 
 class StartDjangoTests(SimpleTestCase):
-    @mock.patch("django.utils.autoreload.StatReloader")
-    def test_watchman_becomes_unavailable(self, mocked_stat):
-        mocked_stat.should_stop.return_value = True
-        fake_reloader = mock.MagicMock()
-        fake_reloader.should_stop = False
-        fake_reloader.run.side_effect = autoreload.WatchmanUnavailable()
-
-        autoreload.start_django(fake_reloader, lambda: None)
-        self.assertEqual(mocked_stat.call_count, 1)
-
     @mock.patch("django.utils.autoreload.ensure_echo_on")
     def test_echo_on_called(self, mocked_echo):
         fake_reloader = mock.MagicMock()
