@@ -48,7 +48,7 @@ class ModelToValidate(models.Model):
 
 class UniqueFieldsModel(models.Model):
     unique_charfield = models.CharField(max_length=100, unique=True)
-    unique_integerfield = models.IntegerField(unique=True)
+    unique_integerfield = models.IntegerField(unique=True, db_default=42)
     non_unique_field = models.IntegerField()
 
 
@@ -161,3 +161,73 @@ class UniqueFuncConstraintModel(models.Model):
         constraints = [
             models.UniqueConstraint(Lower("field"), name="func_lower_field_uq"),
         ]
+
+
+class Product(models.Model):
+    price = models.IntegerField(null=True)
+    discounted_price = models.IntegerField(null=True)
+
+    class Meta:
+        required_db_features = {
+            "supports_table_check_constraints",
+        }
+        constraints = [
+            models.CheckConstraint(
+                check=models.Q(price__gt=models.F("discounted_price")),
+                name="price_gt_discounted_price_validation",
+            ),
+        ]
+
+
+class ChildProduct(Product):
+    class Meta:
+        required_db_features = {
+            "supports_table_check_constraints",
+        }
+
+
+class UniqueConstraintProduct(models.Model):
+    name = models.CharField(max_length=255)
+    color = models.CharField(max_length=32)
+    rank = models.IntegerField()
+
+    class Meta:
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name", "color"], name="name_color_uniq_validation"
+            ),
+            models.UniqueConstraint(fields=["rank"], name="rank_uniq_validation"),
+        ]
+
+
+class ChildUniqueConstraintProduct(UniqueConstraintProduct):
+    pass
+
+
+class UniqueConstraintConditionProduct(models.Model):
+    name = models.CharField(max_length=255)
+    color = models.CharField(max_length=31, null=True, blank=True)
+
+    class Meta:
+        required_db_features = {"supports_partial_indexes"}
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name"],
+                name="name_without_color_uniq_validation",
+                condition=models.Q(color__isnull=True),
+            ),
+        ]
+
+
+class UniqueConstraintNullsDistinctProduct(models.Model):
+    name = models.CharField(max_length=255, blank=True, null=True)
+
+    class Meta:
+        required_db_features = {"supports_nulls_distinct_unique_constraints"}
+        constraints = [
+            models.UniqueConstraint(
+                fields=["name"],
+                name="name_nulls_not_distinct_uniq",
+                nulls_distinct=False,
+            ),
+        ]
