@@ -106,6 +106,7 @@ VALID_URLS = [
     "ddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd"
     "ddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd"
     "ddddddddddddddddd:password@example.com:8080",
+    "http://userid:password" + "d" * 2000 + "@example.aaaaaaaaaaaaa.com",
     "http://142.42.1.1/",
     "http://142.42.1.1:8080/",
     "http://➡.ws/䨹",
@@ -236,6 +237,7 @@ INVALID_URLS = [
     "aaaaaa.com",
     "http://example.aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
     "aaaaaa",
+    "http://example." + ("a" * 63 + ".") * 1000 + "com",
     "http://aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.aaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
     "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.aaaaaaa"
     "aaaaaaaaaaaaaaaaaaaaaaaaaa.aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.aaaaaaaaaaaaaa"
@@ -291,6 +293,7 @@ TEST_DATA = [
     (validate_email, "example@%s.%s.atm" % ("a" * 63, "b" * 10), None),
     (validate_email, "example@atm.%s" % ("a" * 64), ValidationError),
     (validate_email, "example@%s.atm.%s" % ("b" * 64, "a" * 63), ValidationError),
+    (validate_email, "example@%scom" % (("a" * 63 + ".") * 100), ValidationError),
     (validate_email, None, ValidationError),
     (validate_email, "", ValidationError),
     (validate_email, "abc", ValidationError),
@@ -451,11 +454,39 @@ TEST_DATA = [
     (StepValueValidator(3), 1, ValidationError),
     (StepValueValidator(3), 8, ValidationError),
     (StepValueValidator(3), 9, None),
+    (StepValueValidator(2), 4, None),
+    (StepValueValidator(2, offset=1), 3, None),
+    (StepValueValidator(2, offset=1), 4, ValidationError),
     (StepValueValidator(0.001), 0.55, None),
     (StepValueValidator(0.001), 0.5555, ValidationError),
+    (StepValueValidator(0.001, offset=0.0005), 0.5555, None),
+    (StepValueValidator(0.001, offset=0.0005), 0.555, ValidationError),
     (StepValueValidator(Decimal(0.02)), 0.88, None),
     (StepValueValidator(Decimal(0.02)), Decimal(0.88), None),
     (StepValueValidator(Decimal(0.02)), Decimal(0.77), ValidationError),
+    (StepValueValidator(Decimal(0.02), offset=Decimal(0.01)), Decimal(0.77), None),
+    (StepValueValidator(Decimal(2.0), offset=Decimal(0.1)), Decimal(0.1), None),
+    (
+        StepValueValidator(Decimal(0.02), offset=Decimal(0.01)),
+        Decimal(0.88),
+        ValidationError,
+    ),
+    (StepValueValidator(Decimal("1.2"), offset=Decimal("2.2")), Decimal("3.4"), None),
+    (
+        StepValueValidator(Decimal("1.2"), offset=Decimal("2.2")),
+        Decimal("1.2"),
+        ValidationError,
+    ),
+    (
+        StepValueValidator(Decimal("-1.2"), offset=Decimal("2.2")),
+        Decimal("1.1"),
+        ValidationError,
+    ),
+    (
+        StepValueValidator(Decimal("-1.2"), offset=Decimal("2.2")),
+        Decimal("1.0"),
+        None,
+    ),
     (URLValidator(EXTENDED_SCHEMES), "file://localhost/path", None),
     (URLValidator(EXTENDED_SCHEMES), "git://example.com/", None),
     (
@@ -659,24 +690,24 @@ class TestValidatorEquality(TestCase):
 
     def test_regex_equality(self):
         self.assertEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
         )
         self.assertNotEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
-            RegexValidator(r"^(?:[0-9\.\-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
+            RegexValidator(r"^(?:[0-9.-]*)://"),
         )
         self.assertEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
         )
         self.assertNotEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh", "invalid"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
         )
         self.assertNotEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
         )
 
         self.assertNotEqual(
@@ -690,7 +721,7 @@ class TestValidatorEquality(TestCase):
         )
 
     def test_regex_equality_nocache(self):
-        pattern = r"^(?:[a-z0-9\.\-]*)://"
+        pattern = r"^(?:[a-z0-9.-]*)://"
         left = RegexValidator(pattern)
         re.purge()
         right = RegexValidator(pattern)
